/* original parser id follows */
/* yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93" */
/* (use YYMAJOR/YYMINOR for ifdefs dependent on parser version) */

#define YYBYACC 1
#define YYMAJOR 2
#define YYMINOR 0
#define YYPATCH 20240109

#define YYEMPTY        (-1)
#define yyclearin      (yychar = YYEMPTY)
#define yyerrok        (yyerrflag = 0)
#define YYRECOVERING() (yyerrflag != 0)
#define YYENOMEM       (-2)
#define YYEOF          0
#undef YYBTYACC
#define YYBTYACC 0
#define YYDEBUGSTR YYPREFIX "debug"
#define YYPREFIX "yy"

#define YYPURE 0

#line 32 "filter/grammar.y"

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdint.h>
#include <string.h>
#include <stdarg.h>
#include <inttypes.h>
#include <stdlib.h>
#include <stddef.h>
#include <errno.h>
#include <ctype.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "filter.h"
#include "userio.h"
#include "nfxV3.h"
#include "ipconv.h"
#include "sgregex.h"
#include "ja3/ja3.h"
#include "ja4/ja4.h"
#include "nfdump.h"

#define AnyMask 0xffffffffffffffffLL

const data_t NULLPtr = {NULL};

/*
 * function prototypes
 */
static void yyerror(char *msg);

/* var defs */
extern int 			lineno;
extern char 		*yytext;
extern uint32_t	StartNode;
extern char *FilterFilename;

static ipStack_t ipStack[MAXHOSTS];

static uint32_t ChainHosts(ipStack_t *ipStack, int numIP, int direction);

static int AddIdent(char *ident);

static int AddEngineNum(char *type, uint16_t comp, uint64_t num);

static int AddExporterNum(char *type, uint16_t comp, uint64_t num);

static int AddProto(direction_t direction, char *protoStr, uint64_t protoNum);

static int AddPortNumber(direction_t direction, uint16_t comp, uint64_t port);

static int AddICMP(char *type, uint16_t comp, uint64_t number);

static int AddAsNumber(direction_t direction, uint16_t comp, uint64_t as);

static int AddFlagsNumber(direction_t direction, uint16_t comp, uint64_t flags);

static int AddFlagsString(direction_t direction, char *flags);

static int AddTosNumber(direction_t direction, uint16_t comp, uint64_t tos);

static int AddIPttl(uint16_t comp, uint64_t ttl);

static int AddPackets(direction_t direction, uint16_t comp, uint64_t packets);

static int AddBytes(direction_t direction, uint16_t comp, uint64_t bytes);

static int AddFwdStatNum(uint16_t comp, uint64_t num);

static int AddFwdStatString(char *string);

static int AddIP(direction_t direction, char *IPstr);

static int AddIPlist(direction_t direction, void *IPstr);

static int AddNet(direction_t direction, char *IPstr, char *maskStr);

static int AddNetPrefix(direction_t direction, char *IPstr, uint64_t mask);

static int AddInterfaceNumber(direction_t direction, uint64_t num);

static int AddVlanNumber(direction_t direction, uint64_t num);

static int AddMaskNumber(direction_t direction, uint64_t num);

static int AddFlowDir(direction_t direction, int64_t dirNum);

static int AddMPLS(char *type, uint16_t comp, uint64_t value);

static int AddMAC(direction_t direction, char *macString);

static int AddEthertype(uint64_t etherType);

static int AddLatency(direction_t direction, uint16_t comp, uint64_t number);

static int AddASAString(char *event, char *asaStr);

static int AddASA(char *event, uint16_t comp, uint64_t number);

static int AddASApblock(direction_t direction, char *arg);

static int AddNATString(char *event, char *asaStr);

static int AddNAT(char *event, uint16_t comp, uint64_t number);

static int AddNatPortBlocks(char *type, char *subtype, uint16_t comp, uint64_t number);

static int AddACL(direction_t direction, uint16_t comp, uint64_t number);

static int AddPayload(char *type, char *arg, char *opt);

static int AddGeo(direction_t direction, char *geo);

static int AddObservation(char *type, char *subType, uint16_t comp, uint64_t number);

static int AddVRF(direction_t direction, uint16_t comp, uint64_t number);

static int AddPFString(char *type, char *arg);

static int AddPFNumber(char *type, uint16_t comp, uint64_t number);

static void *NewIplist(char *IPstr, int prefix);

static void *NewU64list(uint64_t num);

static int InsertIPlist(void *IPlist, char *IPstr, int64_t prefix);

static int InsertU64list(void *U64list, uint64_t num);

static int AddPortList(direction_t direction, void *U64List);

static int AddASList(direction_t direction, void *U64List);

#ifdef YYSTYPE
#undef  YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#endif
#ifndef YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#line 169 "filter/grammar.y"
typedef union YYSTYPE {
	uint64_t			value;
	char					*s;
	FilterParam_t	param;
	void					*list;
} YYSTYPE;
#endif /* !YYSTYPE_IS_DECLARED */
#line 174 "filter/grammar.c"

/* compatibility with bison */
#ifdef YYPARSE_PARAM
/* compatibility with FreeBSD */
# ifdef YYPARSE_PARAM_TYPE
#  define YYPARSE_DECL() yyparse(YYPARSE_PARAM_TYPE YYPARSE_PARAM)
# else
#  define YYPARSE_DECL() yyparse(void *YYPARSE_PARAM)
# endif
#else
# define YYPARSE_DECL() yyparse(void)
#endif

/* Parameters sent to lex. */
#ifdef YYLEX_PARAM
# define YYLEX_DECL() yylex(void *YYLEX_PARAM)
# define YYLEX yylex(YYLEX_PARAM)
#else
# define YYLEX_DECL() yylex(void)
# define YYLEX yylex()
#endif

#if !(defined(yylex) || defined(YYSTATE))
int YYLEX_DECL();
#endif

/* Parameters sent to yyerror. */
#ifndef YYERROR_DECL
#define YYERROR_DECL() yyerror(const char *s)
#endif
#ifndef YYERROR_CALL
#define YYERROR_CALL(msg) yyerror(msg)
#endif

extern int YYPARSE_DECL();

#define EQ 257
#define LT 258
#define GT 259
#define LE 260
#define GE 261
#define ANY 262
#define NOT 263
#define IDENT 264
#define COUNT 265
#define IP 266
#define IPV4 267
#define IPV6 268
#define IPTTL 269
#define NET 270
#define SRC 271
#define DST 272
#define IN 273
#define OUT 274
#define PREV 275
#define NEXT 276
#define BGP 277
#define ROUTER 278
#define INGRESS 279
#define EGRESS 280
#define CLIENT 281
#define SERVER 282
#define NAT 283
#define XLATE 284
#define TUN 285
#define ENGINE 286
#define ENGINETYPE 287
#define ENGINEID 288
#define EXPORTER 289
#define DURATION 290
#define PPS 291
#define BPS 292
#define BPP 293
#define FLAGS 294
#define PROTO 295
#define PORT 296
#define AS 297
#define IF 298
#define VLAN 299
#define MPLS 300
#define MAC 301
#define ICMP 302
#define ICMPTYPE 303
#define ICMPCODE 304
#define PACKETS 305
#define BYTES 306
#define FLOWS 307
#define ETHERTYPE 308
#define MASK 309
#define FLOWDIR 310
#define TOS 311
#define FWDSTAT 312
#define LATENCY 313
#define ASA 314
#define ACL 315
#define PAYLOAD 316
#define VRF 317
#define OBSERVATION 318
#define PF 319
#define STRING 320
#define GEOSTRING 321
#define NUMBER 322
#define OR 323
#define AND 324
#define NEGATE 325
#define YYERRCODE 256
typedef int YYINT;
static const YYINT yylhs[] = {                           -1,
    0,    0,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    5,    5,    5,    5,    5,    6,
    6,    6,    4,    4,    4,    4,    4,    4,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    1,    1,    1,    1,    1,
};
static const YYINT yylen[] = {                            2,
    0,    1,    1,    1,    1,    2,    3,    3,    3,    4,
    4,    3,    3,    3,    4,    3,    3,    4,    4,    3,
    4,    3,    3,    2,    3,    3,    3,    3,    4,    3,
    4,    3,    4,    5,    3,    3,    4,    3,    2,    2,
    2,    4,    4,    3,    4,    3,    3,    4,    5,    4,
    3,    5,    4,    3,    4,    2,    5,    4,    3,    4,
    3,    6,    6,    6,    1,    3,    2,    3,    4,    1,
    2,    3,    0,    1,    1,    1,    1,    1,    0,    1,
    1,    2,    2,    2,    2,    1,    1,    1,    1,    2,
    2,    2,    2,    1,    1,    1,    1,    1,    1,    2,
    1,    1,    1,    3,    3,    2,    3,
};
static const YYINT yydefred[] = {                         0,
    3,    0,    0,    0,    4,    5,    0,    0,    0,    0,
    0,   98,   99,    0,  101,   94,   95,   96,   97,    0,
   87,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  103,  106,    6,   74,   75,
   76,   77,   78,    0,    0,   82,   84,   83,   85,   90,
   91,   92,   93,  100,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   39,
   86,  102,   40,   41,   24,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   56,
    7,   22,    0,    0,    0,    8,    9,    0,   25,   26,
   27,   28,    0,    0,    0,   16,   17,   30,   23,   46,
   47,    0,    0,    0,   59,   61,    0,  107,    0,  105,
    0,   32,    0,   20,    0,   14,   13,   12,    0,    0,
    0,    0,   35,   36,   44,    0,    0,   38,    0,    0,
    0,    0,    0,   53,   10,   11,   43,   42,   18,   48,
   55,    0,   60,    0,   33,    0,   19,    0,    0,   15,
    0,   37,   29,   31,   21,   45,   50,   58,   52,   57,
    0,    0,   34,   49,   70,    0,    0,    0,    0,   62,
    0,   71,   63,    0,   64,   66,    0,   68,   72,   69,
};
#if defined(YYDESTRUCT_CALL) || defined(YYSTYPE_TOSTRING)
static const YYINT yystos[] = {                           0,
  262,  263,  264,  265,  267,  268,  269,  271,  272,  273,
  274,  275,  276,  277,  278,  279,  280,  281,  282,  283,
  285,  286,  287,  288,  289,  290,  291,  292,  293,  300,
  302,  303,  304,  307,  308,  310,  312,  314,  316,  318,
  319,   40,  327,  328,  329,  330,  328,  320,  257,  258,
  259,  260,  261,  331,  331,  283,  285,  283,  285,  271,
  272,  271,  272,  276,  320,  320,  331,  331,  320,  331,
  331,  331,  331,  262,  320,  320,  331,  331,  331,  322,
  283,  289,  322,  329,  320,  331,  320,  320,  320,  320,
  328,  323,  324,  266,  270,  294,  295,  296,  297,  298,
  299,  301,  305,  306,  309,  311,  313,  315,  317,  321,
  322,  322,  320,  331,  331,  322,  322,  331,  322,  322,
  322,  322,  331,  331,  331,  322,  322,  322,  322,  320,
  329,  331,  320,  320,  320,  329,  331,   41,  328,  328,
  273,  320,  320,  320,  331,  302,  320,  322,  273,  331,
  273,  331,  322,  322,  320,  331,  331,  322,  331,  331,
  331,  331,  331,  322,  322,  322,  322,  322,  322,  322,
  320,  331,  322,   91,  320,   47,  322,  283,   91,  322,
   91,  322,  322,  322,  322,  322,  322,  322,  322,  322,
  320,  332,  322,  320,  322,  333,  333,   47,  320,   93,
   44,  322,   93,   44,   93,  322,   47,  320,  322,  322,
};
#endif /* YYDESTRUCT_CALL || YYSTYPE_TOSTRING */
static const YYINT yydgoto[] = {                         43,
   44,   45,   46,   54,  192,  196,
};
static const YYINT yysindex[] = {                       -38,
    0,  -38, -309, -163,    0,    0, -163, -253, -249, -266,
 -243,    0,    0, -239,    0,    0,    0,    0,    0, -299,
    0, -287, -163, -163, -281, -163, -163, -163, -163, -252,
 -279, -163, -163, -163, -276,  -66, -234, -273, -272, -261,
 -254,  -38,    0, -280, -127,    0,    0,    0,    0,    0,
    0,    0,    0, -251, -241,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -207, -163, -240, -235, -163, -221,
 -220, -218, -211, -163, -163, -163, -210, -208, -204,    0,
    0,    0,    0,    0,    0, -203,  202, -247, -205,  235,
  -41,  -38,  -38, -255, -198, -168, -282, -152, -128, -199,
 -194, -196, -163, -163, -188, -163, -163, -163, -163,    0,
    0,    0, -163, -187, -186,    0,    0, -182,    0,    0,
    0,    0, -180, -176, -174,    0,    0,    0,    0,    0,
    0, -171, -167, -163,    0,    0, -166,    0, -162,    0,
   70,    0,  -30,    0, -159,    0,    0,    0,  -88, -157,
   75, -149,    0,    0,    0, -147, -146,    0, -145, -142,
 -141, -139, -135,    0,    0,    0,    0,    0,    0,    0,
    0, -133,    0, -129,    0, -130,    0, -124, -125,    0,
 -125,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  146,  -32,    0,    0,    0,  -37,  -36, -123,  151,    0,
 -120,    0,    0, -121,    0,    0, -119,    0,    0,    0,
};
static const YYINT yyrindex[] = {                       137,
    0,  231,    0, -118,    0,    0, -118,    1,   35,   69,
  103,    0,    0,    0,    0,    0,    0,    0,    0,  265,
    0,    0, -118, -118,  299, -118, -118, -118, -118,    0,
    0, -118, -118, -118,    0,   14, -118,    0,    0,    0,
    0,  231,    0,  218,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -118, -118,    0,    0, -118,    0,
    0,    0,    0, -118, -118, -118,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    4,    0,    0,    4,
    0,  231,  231,    0,    0, -118,    0, -118, -118,    0,
    0,    0, -118, -118,    0, -118, -118, -118, -118,    0,
    0,    0,   31,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   19, -118,    0,    0,    0,    0,   22,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  -31,    0,    0,    0,    0,    0,    0,    0,  -29,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
};
#if YYBTYACC
static const YYINT yycindex[] = {                         0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
};
#endif
static const YYINT yygindex[] = {                         0,
    7,  -20,    0,   51,    0,   21,
};
#define YYTABLESIZE 620
static const YYINT yytable[] = {                        138,
   80,   42,  179,   79,   60,   61,  204,  204,   47,   74,
   48,  201,   65,   79,   67,   84,  176,  141,   54,  146,
   65,  104,   49,   50,   51,   52,   53,   62,   63,   56,
   51,   57,   66,   58,   81,   59,   64,  147,   69,  148,
   76,   80,   92,   93,   79,   80,   87,   88,   91,   49,
   50,   51,   52,   53,   79,  203,  205,   55,   89,   54,
  200,   65,  104,   67,  142,   90,  131,   75,   88,  136,
  111,   51,  133,   67,   68,   81,   70,   71,   72,   73,
  112,  116,   77,   78,   79,   85,  117,   86,   49,   50,
   51,   52,   53,   49,   50,   51,   52,   53,  139,  140,
  119,  120,   89,  121,   49,   50,   51,   52,   53,   88,
  122,  126,  113,  127,  134,  114,  115,  128,  129,  118,
  149,  143,  153,  155,  123,  124,  125,  154,   49,   50,
   51,   52,   53,  158,  164,  165,    1,  132,   94,  166,
  137,  167,   95,   89,  151,  168,  145,  169,  150,  152,
  170,  144,  171,  156,  157,  173,  159,  160,  161,  162,
  174,   93,  177,  163,  180,  181,   96,   97,   98,   99,
  100,  101,  182,  102,  183,  184,  185,  103,  104,  186,
  187,  105,  188,  106,  172,  107,  189,  108,  190,  109,
  191,  193,  198,  110,  178,  194,  195,  207,  206,  208,
  209,  197,  210,   73,    8,    9,   10,   11,   12,   13,
   14,   15,   16,   17,   18,   19,   81,    2,   21,    0,
    0,    0,   82,    1,    2,    3,    4,    0,    5,    6,
    7,    0,    8,    9,   10,   11,   12,   13,   14,   15,
   16,   17,   18,   19,   20,    0,   21,   22,   23,   24,
   25,   26,   27,   28,   29,   83,    0,    0,    0,    0,
    0,   30,    0,   31,   32,   33,   80,    0,   34,   35,
   80,   36,    0,   37,    0,   38,    0,   39,    0,   40,
   41,   92,   93,    0,  202,  202,    0,  199,   65,  175,
   67,    0,    0,    0,   80,   80,   80,   80,   80,   80,
   81,   80,    0,    0,   81,   80,   80,    0,    0,   80,
    0,   80,    0,   80,    0,   80,    0,   80,    0,    0,
    0,   80,    0,   80,   80,   73,   79,   79,   81,   81,
   81,   81,   81,   81,   88,   81,   79,   79,   88,   81,
   81,   54,   54,   81,  104,   81,    0,   81,    0,   81,
    0,   81,   73,   51,   51,   81,    0,   81,   81,    0,
    0,    0,   88,   88,   88,   88,   88,   88,   89,   88,
    0,    0,   89,   88,   88,    0,    0,   88,    0,   88,
    0,   88,    0,   88,    0,   88,    0,    0,    0,   88,
    0,   88,   88,    0,    0,    0,   89,   89,   89,   89,
   89,   89,   79,   89,    0,    0,   79,   89,   89,    0,
    0,   89,    0,   89,    0,   89,    0,   89,    0,   89,
    0,    0,    0,   89,    0,   89,   89,    0,    0,    0,
   79,   79,   79,   79,   79,   79,    0,   79,    0,    0,
    0,   79,   79,    0,    0,   79,    0,   79,    0,   79,
    0,   79,    0,   79,    0,    0,    0,   79,   49,   50,
   51,   52,   53,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    8,    9,   10,   11,   12,   13,   14,   15,
   16,   17,   18,   19,   81,    0,   21,    0,    0,    0,
   82,   49,   50,   51,   52,   53,   79,    0,    0,    0,
   79,    0,    0,    0,    0,    8,    9,   10,   11,   12,
   13,   14,   15,   16,   17,   18,   19,   81,    0,   21,
    0,  130,    0,   82,   79,   79,   79,   79,   79,   79,
   86,   79,    0,    0,   86,   79,   79,    0,    0,   79,
    0,   79,    0,   79,    0,   79,    0,   79,    0,    0,
    0,   79,    0,    0,  135,    0,    0,    0,   86,   86,
   86,   86,   86,   86,  102,   86,    0,    0,  102,   86,
   86,    0,    0,   86,    0,   86,    0,   86,    0,   86,
    0,   86,    0,    0,    0,   86,    0,    0,    0,    0,
    0,    0,  102,  102,  102,  102,  102,  102,    0,  102,
    0,    0,    0,  102,  102,    0,    0,  102,    0,  102,
    0,  102,    0,  102,    0,  102,    0,    0,    0,  102,
};
static const YYINT yycheck[] = {                         41,
    0,   40,   91,    0,  271,  272,   44,   44,    2,  262,
  320,   44,   44,    0,   44,   36,   47,  273,    0,  302,
  320,    0,  257,  258,  259,  260,  261,  271,  272,  283,
    0,  285,  320,  283,    0,  285,  276,  320,  320,  322,
  320,   41,  323,  324,   41,  322,  320,  320,   42,  257,
  258,  259,  260,  261,   41,   93,   93,    7,  320,   41,
   93,   93,   41,   93,  320,  320,   87,  320,    0,   90,
  322,   41,  320,   23,   24,   41,   26,   27,   28,   29,
  322,  322,   32,   33,   34,  320,  322,   37,  257,  258,
  259,  260,  261,  257,  258,  259,  260,  261,   92,   93,
  322,  322,    0,  322,  257,  258,  259,  260,  261,   41,
  322,  322,  320,  322,  320,   65,   66,  322,  322,   69,
  273,  320,  322,  320,   74,   75,   76,  322,  257,  258,
  259,  260,  261,  322,  322,  322,    0,   87,  266,  322,
   90,  322,  270,   41,  273,  322,   96,  322,   98,   99,
  322,  320,  320,  103,  104,  322,  106,  107,  108,  109,
   91,  324,  322,  113,  322,   91,  294,  295,  296,  297,
  298,  299,  322,  301,  322,  322,  322,  305,  306,  322,
  322,  309,  322,  311,  134,  313,  322,  315,  322,  317,
  320,  322,   47,  321,  283,  320,  322,   47,  322,  320,
  322,  181,  322,  322,  271,  272,  273,  274,  275,  276,
  277,  278,  279,  280,  281,  282,  283,    0,  285,   -1,
   -1,   -1,  289,  262,  263,  264,  265,   -1,  267,  268,
  269,   -1,  271,  272,  273,  274,  275,  276,  277,  278,
  279,  280,  281,  282,  283,   -1,  285,  286,  287,  288,
  289,  290,  291,  292,  293,  322,   -1,   -1,   -1,   -1,
   -1,  300,   -1,  302,  303,  304,  266,   -1,  307,  308,
  270,  310,   -1,  312,   -1,  314,   -1,  316,   -1,  318,
  319,  323,  324,   -1,  322,  322,   -1,  320,  320,  320,
  320,   -1,   -1,   -1,  294,  295,  296,  297,  298,  299,
  266,  301,   -1,   -1,  270,  305,  306,   -1,   -1,  309,
   -1,  311,   -1,  313,   -1,  315,   -1,  317,   -1,   -1,
   -1,  321,   -1,  323,  324,  322,  323,  324,  294,  295,
  296,  297,  298,  299,  266,  301,  323,  324,  270,  305,
  306,  323,  324,  309,  323,  311,   -1,  313,   -1,  315,
   -1,  317,  322,  323,  324,  321,   -1,  323,  324,   -1,
   -1,   -1,  294,  295,  296,  297,  298,  299,  266,  301,
   -1,   -1,  270,  305,  306,   -1,   -1,  309,   -1,  311,
   -1,  313,   -1,  315,   -1,  317,   -1,   -1,   -1,  321,
   -1,  323,  324,   -1,   -1,   -1,  294,  295,  296,  297,
  298,  299,  266,  301,   -1,   -1,  270,  305,  306,   -1,
   -1,  309,   -1,  311,   -1,  313,   -1,  315,   -1,  317,
   -1,   -1,   -1,  321,   -1,  323,  324,   -1,   -1,   -1,
  294,  295,  296,  297,  298,  299,   -1,  301,   -1,   -1,
   -1,  305,  306,   -1,   -1,  309,   -1,  311,   -1,  313,
   -1,  315,   -1,  317,   -1,   -1,   -1,  321,  257,  258,
  259,  260,  261,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  271,  272,  273,  274,  275,  276,  277,  278,
  279,  280,  281,  282,  283,   -1,  285,   -1,   -1,   -1,
  289,  257,  258,  259,  260,  261,  266,   -1,   -1,   -1,
  270,   -1,   -1,   -1,   -1,  271,  272,  273,  274,  275,
  276,  277,  278,  279,  280,  281,  282,  283,   -1,  285,
   -1,  320,   -1,  289,  294,  295,  296,  297,  298,  299,
  266,  301,   -1,   -1,  270,  305,  306,   -1,   -1,  309,
   -1,  311,   -1,  313,   -1,  315,   -1,  317,   -1,   -1,
   -1,  321,   -1,   -1,  320,   -1,   -1,   -1,  294,  295,
  296,  297,  298,  299,  266,  301,   -1,   -1,  270,  305,
  306,   -1,   -1,  309,   -1,  311,   -1,  313,   -1,  315,
   -1,  317,   -1,   -1,   -1,  321,   -1,   -1,   -1,   -1,
   -1,   -1,  294,  295,  296,  297,  298,  299,   -1,  301,
   -1,   -1,   -1,  305,  306,   -1,   -1,  309,   -1,  311,
   -1,  313,   -1,  315,   -1,  317,   -1,   -1,   -1,  321,
};
#if YYBTYACC
static const YYINT yyctable[] = {                        -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
};
#endif
#define YYFINAL 43
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 325
#define YYUNDFTOKEN 334
#define YYTRANSLATE(a) ((a) > YYMAXTOKEN ? YYUNDFTOKEN : (a))
#if YYDEBUG
static const char *const yyname[] = {

"$end",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,"'('","')'","'*'","'+'","','",0,0,"'/'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'['",0,"']'",0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"error",
"EQ","LT","GT","LE","GE","ANY","NOT","IDENT","COUNT","IP","IPV4","IPV6","IPTTL",
"NET","SRC","DST","IN","OUT","PREV","NEXT","BGP","ROUTER","INGRESS","EGRESS",
"CLIENT","SERVER","NAT","XLATE","TUN","ENGINE","ENGINETYPE","ENGINEID",
"EXPORTER","DURATION","PPS","BPS","BPP","FLAGS","PROTO","PORT","AS","IF","VLAN",
"MPLS","MAC","ICMP","ICMPTYPE","ICMPCODE","PACKETS","BYTES","FLOWS","ETHERTYPE",
"MASK","FLOWDIR","TOS","FWDSTAT","LATENCY","ASA","ACL","PAYLOAD","VRF",
"OBSERVATION","PF","STRING","GEOSTRING","NUMBER","OR","AND","NEGATE","$accept",
"prog","expr","dqual","term","comp","iplist","u64list","illegal-symbol",
};
static const char *const yyrule[] = {
"$accept : prog",
"prog :",
"prog : expr",
"term : ANY",
"term : IPV4",
"term : IPV6",
"term : IDENT STRING",
"term : COUNT comp NUMBER",
"term : ENGINETYPE comp NUMBER",
"term : ENGINEID comp NUMBER",
"term : ENGINE STRING comp NUMBER",
"term : EXPORTER STRING comp NUMBER",
"term : dqual PROTO NUMBER",
"term : dqual PROTO STRING",
"term : dqual PROTO ICMP",
"term : dqual PORT comp NUMBER",
"term : ICMPTYPE comp NUMBER",
"term : ICMPCODE comp NUMBER",
"term : ICMP STRING comp NUMBER",
"term : dqual FLAGS comp NUMBER",
"term : dqual FLAGS STRING",
"term : dqual TOS comp NUMBER",
"term : IPTTL comp NUMBER",
"term : FWDSTAT comp NUMBER",
"term : FWDSTAT STRING",
"term : DURATION comp NUMBER",
"term : PPS comp NUMBER",
"term : BPS comp NUMBER",
"term : BPP comp NUMBER",
"term : dqual PACKETS comp NUMBER",
"term : FLOWS comp NUMBER",
"term : dqual BYTES comp NUMBER",
"term : dqual IP STRING",
"term : dqual NET STRING STRING",
"term : dqual NET STRING '/' NUMBER",
"term : dqual IF NUMBER",
"term : dqual VLAN NUMBER",
"term : dqual AS comp NUMBER",
"term : dqual MASK NUMBER",
"term : ETHERTYPE NUMBER",
"term : FLOWDIR NUMBER",
"term : FLOWDIR dqual",
"term : MPLS STRING comp NUMBER",
"term : MPLS ANY comp NUMBER",
"term : dqual MAC STRING",
"term : dqual LATENCY comp NUMBER",
"term : ASA STRING STRING",
"term : ASA STRING dqual",
"term : ASA STRING comp NUMBER",
"term : dqual PORT IN NAT STRING",
"term : dqual ACL comp NUMBER",
"term : NAT STRING STRING",
"term : NAT STRING STRING comp NUMBER",
"term : NAT STRING comp NUMBER",
"term : PAYLOAD STRING STRING",
"term : PAYLOAD STRING STRING STRING",
"term : dqual GEOSTRING",
"term : OBSERVATION STRING STRING comp NUMBER",
"term : dqual VRF comp NUMBER",
"term : PF STRING STRING",
"term : PF STRING comp NUMBER",
"term : PF STRING dqual",
"term : dqual IP IN '[' iplist ']'",
"term : dqual PORT IN '[' u64list ']'",
"term : dqual AS IN '[' u64list ']'",
"iplist : STRING",
"iplist : STRING '/' NUMBER",
"iplist : iplist STRING",
"iplist : iplist ',' STRING",
"iplist : iplist STRING '/' NUMBER",
"u64list : NUMBER",
"u64list : u64list NUMBER",
"u64list : u64list ',' NUMBER",
"comp :",
"comp : EQ",
"comp : LT",
"comp : GT",
"comp : LE",
"comp : GE",
"dqual :",
"dqual : SRC",
"dqual : DST",
"dqual : SRC NAT",
"dqual : DST NAT",
"dqual : SRC TUN",
"dqual : DST TUN",
"dqual : NAT",
"dqual : TUN",
"dqual : IN",
"dqual : OUT",
"dqual : IN SRC",
"dqual : IN DST",
"dqual : OUT SRC",
"dqual : OUT DST",
"dqual : INGRESS",
"dqual : EGRESS",
"dqual : CLIENT",
"dqual : SERVER",
"dqual : PREV",
"dqual : NEXT",
"dqual : BGP NEXT",
"dqual : ROUTER",
"dqual : EXPORTER",
"expr : term",
"expr : expr OR expr",
"expr : expr AND expr",
"expr : NOT expr",
"expr : '(' expr ')'",

};
#endif

#if YYDEBUG
int      yydebug;
#endif

int      yyerrflag;
int      yychar;
YYSTYPE  yyval;
YYSTYPE  yylval;
int      yynerrs;

#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
YYLTYPE  yyloc; /* position returned by actions */
YYLTYPE  yylloc; /* position from the lexer */
#endif

#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
#ifndef YYLLOC_DEFAULT
#define YYLLOC_DEFAULT(loc, rhs, n) \
do \
{ \
    if (n == 0) \
    { \
        (loc).first_line   = YYRHSLOC(rhs, 0).last_line; \
        (loc).first_column = YYRHSLOC(rhs, 0).last_column; \
        (loc).last_line    = YYRHSLOC(rhs, 0).last_line; \
        (loc).last_column  = YYRHSLOC(rhs, 0).last_column; \
    } \
    else \
    { \
        (loc).first_line   = YYRHSLOC(rhs, 1).first_line; \
        (loc).first_column = YYRHSLOC(rhs, 1).first_column; \
        (loc).last_line    = YYRHSLOC(rhs, n).last_line; \
        (loc).last_column  = YYRHSLOC(rhs, n).last_column; \
    } \
} while (0)
#endif /* YYLLOC_DEFAULT */
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */
#if YYBTYACC

#ifndef YYLVQUEUEGROWTH
#define YYLVQUEUEGROWTH 32
#endif
#endif /* YYBTYACC */

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH  10000
#endif
#endif

#ifndef YYINITSTACKSIZE
#define YYINITSTACKSIZE 200
#endif

typedef struct {
    unsigned stacksize;
    YYINT    *s_base;
    YYINT    *s_mark;
    YYINT    *s_last;
    YYSTYPE  *l_base;
    YYSTYPE  *l_mark;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    YYLTYPE  *p_base;
    YYLTYPE  *p_mark;
#endif
} YYSTACKDATA;
#if YYBTYACC

struct YYParseState_s
{
    struct YYParseState_s *save;    /* Previously saved parser state */
    YYSTACKDATA            yystack; /* saved parser stack */
    int                    state;   /* saved parser state */
    int                    errflag; /* saved error recovery status */
    int                    lexeme;  /* saved index of the conflict lexeme in the lexical queue */
    YYINT                  ctry;    /* saved index in yyctable[] for this conflict */
};
typedef struct YYParseState_s YYParseState;
#endif /* YYBTYACC */
/* variables for the parser stack */
static YYSTACKDATA yystack;
#if YYBTYACC

/* Current parser state */
static YYParseState *yyps = 0;

/* yypath != NULL: do the full parse, starting at *yypath parser state. */
static YYParseState *yypath = 0;

/* Base of the lexical value queue */
static YYSTYPE *yylvals = 0;

/* Current position at lexical value queue */
static YYSTYPE *yylvp = 0;

/* End position of lexical value queue */
static YYSTYPE *yylve = 0;

/* The last allocated position at the lexical value queue */
static YYSTYPE *yylvlim = 0;

#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
/* Base of the lexical position queue */
static YYLTYPE *yylpsns = 0;

/* Current position at lexical position queue */
static YYLTYPE *yylpp = 0;

/* End position of lexical position queue */
static YYLTYPE *yylpe = 0;

/* The last allocated position at the lexical position queue */
static YYLTYPE *yylplim = 0;
#endif

/* Current position at lexical token queue */
static YYINT  *yylexp = 0;

static YYINT  *yylexemes = 0;
#endif /* YYBTYACC */
#line 560 "filter/grammar.y"

#define EBUFFSIZE 512
static char ebuf[EBUFFSIZE];

static void yyerror(char *msg) {
	if ( FilterFilename ) {
		printf("File '%s' line %d: %s at '%s'\n", FilterFilename, lineno, msg, yytext);
	} else {
		printf("Line %d: %s at '%s'\n", lineno, msg, yytext);
	}
} /* End of     */

#define yyprintf(...) do { \
    snprintf(ebuf, EBUFFSIZE, __VA_ARGS__); \
    yyerror(ebuf); \
} while (0)

static uint32_t NewIPElement(ipStack_t *ipStack, int direction, int comp, data_t *data) {

	int block = -1;

	if ( ipStack->af == PF_INET ) {
		// handle IPv4 addr element
		switch ( direction ) {
			case DIR_SRC:
				block = NewElement(EXipv4FlowID, OFFsrc4Addr, SIZEsrc4Addr, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
			case DIR_DST:
				block = NewElement(EXipv4FlowID, OFFdst4Addr, SIZEdst4Addr, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
			case DIR_SRC_NAT:
				block = NewElement(EXnatXlateIPv4ID, OFFxlateSrc4Addr, SIZExlateSrc4Addr, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
			case DIR_DST_NAT:
				block = NewElement(EXnatXlateIPv4ID, OFFxlateDst4Addr, SIZExlateDst4Addr, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
			case DIR_SRC_TUN:
				block = NewElement(EXtunIPv4ID, OFFtunSrc4Addr, SIZEtunSrc4Addr, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
			case DIR_DST_TUN:
				block = NewElement(EXtunIPv4ID, OFFtunDst4Addr, SIZEtunDst4Addr, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
			case DIR_NEXT:
				block = NewElement(EXipNextHopV4ID, OFFNextHopV4IP, SIZENextHopV4IP, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
			case BGP_NEXT:
				block = NewElement(EXbgpNextHopV4ID, OFFbgp4NextIP, SIZEbgp4NextIP, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
			case SRC_ROUTER:
				block = NewElement(EXipReceivedV4ID, OFFReceived4IP, SIZEReceived4IP, ipStack->ipaddr[1], comp, FUNC_NONE, data[0]); 
				break;
		} // End of switch

	} else {
		// handle IPv6 addr element
		int v6_1, v6_2 = 0;
		switch ( direction ) {
			case DIR_SRC:
				v6_1 = NewElement(EXipv6FlowID, OFFsrc6Addr, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXipv6FlowID, OFFsrc6Addr + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
			case DIR_DST:
				v6_1 = NewElement(EXipv6FlowID, OFFdst6Addr, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXipv6FlowID, OFFdst6Addr + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
			case DIR_SRC_NAT:
				v6_1 = NewElement(EXnatXlateIPv6ID, OFFxlateSrc6Addr, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXnatXlateIPv6ID, OFFxlateSrc6Addr + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
			case DIR_DST_NAT:
				v6_1 = NewElement(EXnatXlateIPv6ID, OFFxlateDst6Addr, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXnatXlateIPv6ID, OFFxlateDst6Addr + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
			case DIR_SRC_TUN:
				v6_1 = NewElement(EXtunIPv6ID, OFFtunSrc6Addr, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXtunIPv6ID, OFFtunSrc6Addr + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
			case DIR_DST_TUN:
				v6_1 = NewElement(EXtunIPv6ID, OFFtunDst6Addr, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXtunIPv6ID, OFFtunDst6Addr + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
			case DIR_NEXT:
				v6_1 = NewElement(EXipNextHopV6ID, OFFNextHopV6IP, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXipNextHopV6ID, OFFNextHopV6IP + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
			case BGP_NEXT:
				v6_1 = NewElement(EXbgpNextHopV6ID, OFFbgp6NextIP, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXbgpNextHopV6ID, OFFbgp6NextIP + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
			case SRC_ROUTER:
				v6_1 = NewElement(EXipReceivedV6ID, OFFReceived6IP, sizeof(uint64_t), ipStack->ipaddr[0], comp, FUNC_NONE, data[0]);
				v6_2 = NewElement(EXipReceivedV6ID, OFFReceived6IP + sizeof(uint64_t), sizeof(uint64_t), ipStack->ipaddr[1], comp, FUNC_NONE, data[1]);
				break;
		} // End of switch

		// IPv6
		if ( v6_1 && v6_2 )
			block = Connect_AND(v6_1, v6_2);

	}

	return block;
} // NewIPElement

static uint32_t ChainHosts(ipStack_t *ipStack, int numIP, int direction) {

	data_t data[2] = { NULLPtr, NULLPtr };

	uint32_t final = 0;
	int i = 0;
	do {
		// chain multiple IPs
		int block = NewIPElement(&ipStack[i], direction, CMP_EQ, data);
		final = final == 0 ? block : Connect_OR(final, block);

	} while (++i < numIP);
	
	return final;
} // End of ChainHosts

static int AddIdent(char *ident) {
	char *c;

	// ident[a-zA-Z0-9_\-]+ { 
	size_t len = strlen(ident);
	if ( len == 0 || len > 255 ) {
		yyprintf("Invalid ident string: %s", ident);
		return -1;
	}
	
	c = &ident[0];
	while ( *c ) {
		if ( *c != '_' && *c != '-' && !isalnum(*c) ) {
			yyprintf("Invalid char in ident string: %s: %c", ident, *c);
			return 0;
		}
		c++;
	}
	
	data_t data = {.dataPtr = strdup(ident)};
	return NewElement(EXnull, 0, 0, 0, CMP_IDENT, FUNC_NONE, data); 

} // End of AddIdent

static int AddProto(direction_t direction, char *protoStr, uint64_t protoNum) {

	if ( protoNum > 255 ) {
		yyprintf("Protocol %" PRIu64 " out of range", protoNum);
		return -1;
	}

	if ( protoStr != NULL ) {
		protoNum = ProtoNum(protoStr);
  	if ( protoNum == -1 ) {
	  	yyprintf("Unknown protocol: %s", protoStr);
			Protoinfo(protoStr);
			return -1;
  	}
	}

	if ( direction == DIR_UNSPEC ) {
		return NewElement(EXgenericFlowID, OFFproto, SIZEproto, protoNum, CMP_EQ, FUNC_NONE, NULLPtr); 
	} else if ( direction == DIR_UNSPEC_TUN ) {
		return Connect_OR(
			NewElement(EXtunIPv4ID, OFFtunProtoV4, SIZEtunProtoV4, protoNum, CMP_EQ, FUNC_NONE, NULLPtr),
			NewElement(EXtunIPv6ID, OFFtunProtoV6, SIZEtunProtoV6, protoNum, CMP_EQ, FUNC_NONE, NULLPtr)
		);
	} else {
	  	yyprintf("Unknown protocol specifier");
			return -1;
	}
} // End of AddProtoString

static int AddEngineNum(char *type, uint16_t comp, uint64_t num) {
	if ( num > 255 ) {
		yyprintf("Engine argument %" PRIu64 " of range 0..255", num);
		return -1;
  }

	int ret = -1;
	if ( strcasecmp(type, "type") == 0 ) {
		ret = NewElement(EXnull, OFFengineType, SIZEengineType, num, comp, FUNC_NONE, NULLPtr);
	} else if ( strcasecmp(type, "id") == 0 ) {
		ret = NewElement(EXnull, OFFengineID, SIZEengineID, num, comp, FUNC_NONE, NULLPtr);
	}

	return ret;
} // End of AddEngineNum

static int AddExporterNum(char *type, uint16_t comp, uint64_t num) {
	if ( num > 65535 ) {
	  yyprintf("Exporter argument %" PRIu64 " of range 0..65535", num);
		return -1;
	}

	int ret = -1;
  if ((strcasecmp(type, "id") == 0 ) || (strcasecmp(type, "sysid") == 0)) {
		ret = NewElement(EXnull, OFFexporterID, SIZEexporterID, num, comp, FUNC_NONE, NULLPtr);
	} else {
	  yyprintf("Unknown exporter argument: %s", type);
	}

	return ret;
} // End of AddExporterNum

static int AddPortNumber(direction_t direction, uint16_t comp, uint64_t port) {
	if ( port > 65535 ) {
		  yyprintf("Port number: %" PRIu64 " out of range", port);
			return -1;
	}

	int ret = -1;
  switch ( direction ) {
	  case DIR_SRC:
		  ret = NewElement(EXgenericFlowID, OFFsrcPort, SIZEsrcPort, port, comp, FUNC_NONE, NULLPtr);
		  break;
	  case DIR_DST:
		  ret = NewElement(EXgenericFlowID, OFFdstPort, SIZEdstPort, port, comp, FUNC_NONE, NULLPtr);
		  break;
	  case DIR_SRC_NAT:
		  ret = NewElement(EXnatXlatePortID, OFFxlateSrcPort, SIZExlateSrcPort, port, comp, FUNC_NONE, NULLPtr);
		  break;
	  case DIR_DST_NAT:
		  ret = NewElement(EXnatXlatePortID, OFFxlateDstPort, SIZExlateDstPort, port, comp, FUNC_NONE, NULLPtr);
		  break;
	  case DIR_UNSPEC:
		  ret = Connect_OR(
			  NewElement(EXgenericFlowID, OFFsrcPort, SIZEsrcPort, port, comp, FUNC_NONE, NULLPtr),
			  NewElement(EXgenericFlowID, OFFdstPort, SIZEdstPort, port, comp, FUNC_NONE, NULLPtr)
		  );
		  break;
	  case DIR_UNSPEC_NAT:
		  ret = Connect_OR(
		  	NewElement(EXnatXlatePortID, OFFxlateSrcPort, SIZExlateSrcPort, port, comp, FUNC_NONE, NULLPtr),
		  	NewElement(EXnatXlatePortID, OFFxlateDstPort, SIZExlateDstPort, port, comp, FUNC_NONE, NULLPtr)
		  );
		  break;
	  default:
		  yyprintf("Unknown direction");
  } // End switch

	return ret;
} // End of AddPortNumber

static int AddICMP(char *type, uint16_t comp, uint64_t number) {
	if ( number > 255 ) {
		  yyprintf("ICMP argument %" PRIu64 " of range 0..255", number);
			return -1;
  }

	int ret = -1;
	// imply ICMP-TYPE with a proto ICMP block
	int protoICMP = Connect_OR (
			  NewElement(EXgenericFlowID, OFFproto, SIZEproto, IPPROTO_ICMP, CMP_EQ, FUNC_NONE, NULLPtr), 
			  NewElement(EXgenericFlowID, OFFproto, SIZEproto, IPPROTO_ICMPV6, CMP_EQ, FUNC_NONE, NULLPtr)
		  );
	if ( strcasecmp(type, "type") == 0 ) {
		ret = Connect_AND(
			protoICMP,
		  NewElement(EXgenericFlowID, OFFicmpType, SIZEicmpType, number, comp, FUNC_NONE, NULLPtr)
	  );
	} else if ( strcasecmp(type, "code") == 0 ) {
		ret = Connect_AND(
			protoICMP,
			NewElement(EXgenericFlowID, OFFicmpCode, SIZEicmpCode, number, comp, FUNC_NONE, NULLPtr)
	  );
	} 

	return ret;
} // End of AddICMP

static int AddFlagsNumber(direction_t direction, uint16_t comp, uint64_t flags) {
	if ( flags > 255 ) {
		  yyprintf("flags number %" PRIu64 " > 255", flags);
			return -1;
	}

	// direction ignored

	return Connect_AND(
	  // imply flags with proto TCP
	  NewElement(EXgenericFlowID, OFFproto, SIZEproto, IPPROTO_TCP, CMP_EQ, FUNC_NONE, NULLPtr), 
	  NewElement(EXgenericFlowID, OFFtcpFlags, SIZEtcpFlags, flags, comp, FUNC_NONE, NULLPtr)
  );
} // End of AddFlagsNumber

static int AddFlagsString(direction_t direction, char *flags) {
	size_t len = strlen(flags);
  if ( len > 10 ) {
	  yyprintf("Flags string error");
		return -1;
  }

	int strict = 0;
	if ( flags[0] == '=') {
	  strict = 1;
	  len--;
  }

  int cnt     = 0;
  uint64_t fl = 0;
  if ( strchr(flags, 'F') ) { fl |=  1; cnt++; }
  if ( strchr(flags, 'S') ) { fl |=  2; cnt++; }
  if ( strchr(flags, 'R') ) { fl |=  4; cnt++; }
  if ( strchr(flags, 'P') ) { fl |=  8; cnt++; }
  if ( strchr(flags, 'A') ) { fl |=  16; cnt++; }
  if ( strchr(flags, 'U') ) { fl |=  32; cnt++; }
  if ( strchr(flags, 'E') ) { fl |=  64; cnt++; }
  if ( strchr(flags, 'C') ) { fl |= 128; cnt++; }
  if ( strchr(flags, 'X') ) { fl =  63; cnt++; }

  if ( cnt != len ) {
	  yyprintf("Unknown flags");
		return -1;
  }

  if (strict) {
		return AddFlagsNumber(direction, CMP_EQ, fl);
  } else {
		return AddFlagsNumber(direction, CMP_FLAGS, fl);
  }

	// unreached
} // End of AddFlagsString

static int AddTosNumber(direction_t direction, uint16_t comp, uint64_t tos) {
	if ( tos > 255 ) {
		yyprintf("Tos number %" PRIu64 " out of range", tos);
		return -1;
  }

	int ret = -1;
  switch (direction) {
	  case DIR_UNSPEC:
	  case DIR_SRC: 
		  ret = NewElement(EXgenericFlowID, OFFsrcTos, SIZEsrcTos, tos, comp, FUNC_NONE, NULLPtr);
		  break;
	  case DIR_DST: 
		  ret = NewElement(EXflowMiscID, OFFdstTos, SIZEdstTos, tos, comp, FUNC_NONE, NULLPtr);
		  break;
	  default:
		  yyprintf("syntax error");
  } // End of switch

	return ret;
} // End of AddTosNumber

static int AddIPttl(uint16_t comp, uint64_t ttl) {
	if ( ttl > 255 ) {
		yyprintf("TTL number out of range");
		return -1;
  }

	return NewElement(EXipInfoID, OFFipTTL, SIZEipTTL, ttl, comp, FUNC_NONE, NULLPtr);
} // End of AddIPttl

static int AddPackets(direction_t direction, uint16_t comp, uint64_t packets) {

	int ret = -1;
	switch ( direction ) {
		case DIR_UNSPEC:
	  case DIR_IN: 
		  ret = NewElement(EXgenericFlowID, OFFinPackets, SIZEinPackets, packets, comp, FUNC_NONE, NULLPtr); 
		  break;
	  case DIR_OUT: 
		  ret = NewElement(EXgenericFlowID, OFFoutPackets, SIZEoutPackets, packets, comp, FUNC_NONE, NULLPtr); 
		break;
	  default:
		  yyprintf("Invalid direction for packets");
	} // End of switch
	return ret;
} // End of AddPackets

static int AddBytes(direction_t direction, uint16_t comp, uint64_t bytes) {
	int ret = -1;
	switch ( direction ) {
	  case DIR_UNSPEC:
	  case DIR_IN: 
		  ret = NewElement(EXgenericFlowID, OFFinBytes, SIZEinBytes, bytes, comp, FUNC_NONE, NULLPtr); 
		  break;
	  case DIR_OUT: 
		  ret = NewElement(EXgenericFlowID, OFFoutBytes, SIZEoutBytes, bytes, comp, FUNC_NONE, NULLPtr); 
		  break;
	  default:
		  yyprintf("Invalid direction for bytes");
	 } // End of switch
	 return ret;
} // End of AddBytes

static int AddFwdStatNum(uint16_t comp, uint64_t num) {
	if ( num > 255 ) {
	  yyprintf("Forwarding status: %" PRIu64 " our of range", num);
		return -1;
	}

	return NewElement(EXgenericFlowID, OFFfwdStatus, SIZEfwdStatus, num, comp, FUNC_NONE, NULLPtr);
} // End of AddFwdStatNum

static int AddFwdStatString(char *string) {
	int	fwdStatus = fwdStatusNum(string);
	if ( fwdStatus < 0 ) {
	  fwdStatusInfo();
	  yyprintf("Unkown forwarding status: %s", string);
		return -1;
	}

	return NewElement(EXgenericFlowID, OFFfwdStatus, SIZEfwdStatus, fwdStatus, CMP_EQ, FUNC_NONE, NULLPtr);
} // End of AddFwdStatString

static int AddMPLS(char *type, uint16_t comp, uint64_t value) {
	if ( strncasecmp(type, "label", 5) == 0 ) {
		char *s = type + 5;
		if ( *s == '\0' ) {
			yyprintf("Missing mpls stack number for label");
			return -1;
		}
		int lnum = (int)strtol(s, (char **)NULL, 10);
		data_t labelIndex = { .dataVal = lnum};
		return NewElement(EXmplsLabelID, 0, 0, value, comp, FUNC_MPLS_LABEL, labelIndex);
	} else if ( strcasecmp(type, "any") == 0 ) {
		data_t labelValue = { .dataVal = value};
		return NewElement(EXmplsLabelID, 0, 0, value, comp, FUNC_MPLS_ANY, labelValue);
	} else if ( strcasecmp(type, "eos") == 0 ) {
		// match End of Stack label 
		return NewElement(EXmplsLabelID, 0, 0, value, comp, FUNC_MPLS_EOS, NULLPtr);
	} else if ( strncasecmp(type, "exp", 3) == 0 ) {
		char *s = type + 3;
		if ( *s == '\0' ) {
			yyprintf("Missing mpls stack number for exp value");
			return -1;
		}
		int lnum = (int)strtol(s, (char **)NULL, 10);
		data_t data = {.dataVal = lnum};
		return NewElement(EXmplsLabelID, 0, 0, value, comp, FUNC_MPLS_EXP, data);
	} else {
			yyprintf("Unknown mpls argument: %s", type);
			return -1;
	}

	// unreached
	return -1;
} // End of AddMPLS

static int AddEthertype(uint64_t etherType) {
	return NewElement(EXlayer2ID, OFFetherType, SIZEetherType, etherType, CMP_EQ, FUNC_NONE, NULLPtr);
} // End of AddMAC

static int AddMAC(direction_t direction, char *macString) {

	uint64_t macVal = Str2Mac(macString);
	if ( macVal == 0 ) return -1;

	switch (direction) {
		case DIR_IN_SRC:
			return NewElement(EXmacAddrID, OFFinSrcMac, SIZEinSrcMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		case DIR_IN_DST:
			return NewElement(EXmacAddrID, OFFinDstMac, SIZEinDstMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		case DIR_OUT_SRC:
			return NewElement(EXmacAddrID, OFFoutSrcMac, SIZEoutSrcMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		case DIR_OUT_DST:
			return NewElement(EXmacAddrID, OFFoutDstMac, SIZEoutDstMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		case DIR_SRC:
			return Connect_OR (
				NewElement(EXmacAddrID, OFFinSrcMac, SIZEinSrcMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr),
				NewElement(EXmacAddrID, OFFoutSrcMac, SIZEoutSrcMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			break;
		case DIR_DST:
			return Connect_OR (
				NewElement(EXmacAddrID, OFFinDstMac, SIZEinDstMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr),
				NewElement(EXmacAddrID, OFFoutDstMac, SIZEoutDstMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			break;
		case DIR_IN:
			return Connect_OR (
				NewElement(EXmacAddrID, OFFinSrcMac, SIZEinSrcMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr),
				NewElement(EXmacAddrID, OFFinDstMac, SIZEinDstMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			break;
		case DIR_OUT:
			return Connect_OR (
				NewElement(EXmacAddrID, OFFoutSrcMac, SIZEoutSrcMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr),
				NewElement(EXmacAddrID, OFFoutDstMac, SIZEoutDstMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			break;
		case DIR_UNSPEC: {
				int in = Connect_OR (
					NewElement(EXmacAddrID, OFFinSrcMac, SIZEinSrcMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr),
					NewElement(EXmacAddrID, OFFinDstMac, SIZEinDstMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr)
				);
				int out = Connect_OR (
					NewElement(EXmacAddrID, OFFoutSrcMac, SIZEoutSrcMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr),
					NewElement(EXmacAddrID, OFFoutDstMac, SIZEoutDstMac, macVal, CMP_EQ, FUNC_NONE, NULLPtr)
				);
				return Connect_OR(in, out);
			} break;
		default:
			yyprintf("Unknown mac argument");
			return -1;
	}

	// unreached
	return -1;
} // End of AddMAC

static int AddLatency(direction_t direction, uint16_t comp, uint64_t number) {

	int ret = -1;
  switch (direction) {
		case DIR_CLIENT:
			ret =  NewElement(EXlatencyID, OFFusecClientNwDelay, SIZEusecClientNwDelay, number, comp, FUNC_NONE, NULLPtr);
		  break;
		case DIR_SERVER:
			ret =  NewElement(EXlatencyID, OFFusecServerNwDelay, SIZEusecServerNwDelay, number, comp, FUNC_NONE, NULLPtr);
		  break;
		// case XXX	ret =  NewElement(EXlatencyID, OFFusecApplLatency, SIZEusecApplLatency, number, comp, FUNC_NONE, NULLPtr);
		default:
			yyprintf("Unknown latency argument");
  }

	return ret;
} // End of AddLatency

static int AddASAString(char *event, char *asaStr) {

	if (strcasecmp(event, "event") == 0) {
		int eventNum = fwEventID(asaStr);
		if ( eventNum < 0 ) {
			yyprintf("Invalid ASA event type: %s", asaStr);
			return -1;
		}
		return NewElement(EXnselCommonID, OFFfwEvent, SIZEfwEvent, eventNum, CMP_EQ, FUNC_NONE, NULLPtr);
	} else if (strcasecmp(event, "denied") == 0) {
		int eventNum = fwXEventID(asaStr);
		if ( eventNum < 0 ) {
			yyprintf("Invalid ASA Xevent type: %s", asaStr);
			return -1;
		}
		return NewElement(EXnselCommonID, OFFfwXevent, SIZEfwXevent, eventNum, CMP_EQ, FUNC_NONE, NULLPtr);
	} else if (strcasecmp(event, "user") == 0) {
		if ( strlen(asaStr) > 65 ) {
			yyprintf("Length of ASA user name: %s > 65 chars", asaStr);
			return -1;
		}
		data_t data = {.dataPtr = strdup(asaStr)};
		return NewElement(EXnselUserID, OFFusername, 0, 0, CMP_STRING, FUNC_NONE, data);
	}

	yyprintf("Invalid ASA type: %s", event);
	return -1;

} // End of AddASAString

static int AddASA(char *event, uint16_t comp, uint64_t number) {

	if ( strcasecmp(event, "event") == 0 ) {
		if ( number > 5 ) {
			yyprintf("Invalid event number %" PRIu64 ". Expected 0..5", number);
			return -1;
		}
		return NewElement(EXnselCommonID, OFFfwEvent, SIZEfwEvent, number, comp, FUNC_NONE, NULLPtr);
	} else if ( strcasecmp(event, "xevent") == 0 ) {
		return NewElement(EXnselCommonID, OFFfwXevent, SIZEfwXevent, number, comp, FUNC_NONE, NULLPtr);
	}

	yyprintf("Invalid ASA type: %s", event);
	return -1;

} // End of AddASA

static int AddACL(direction_t direction, uint16_t comp, uint64_t number) {

	uint32_t offset = 0;
	switch (direction) {
		case DIR_INGRESS:
			offset = OFFingressAcl;
			break;
		case DIR_EGRESS:
			offset = OFFegressAcl;
			break;
		default:
			yyprintf("Invalid ACL direction");
			return -1;
	}
	
	uint32_t acl[3];
	acl[0] = NewElement(EXnselAclID, offset, sizeof(uint32_t), number, comp, FUNC_NONE, NULLPtr);
	acl[1] = NewElement(EXnselAclID, offset + sizeof(uint32_t), sizeof(uint32_t), number, comp, FUNC_NONE, NULLPtr);
	acl[2] = NewElement(EXnselAclID, offset + 2*sizeof(uint32_t), sizeof(uint32_t), number, comp, FUNC_NONE, NULLPtr);
	return Connect_OR (
		Connect_OR(acl[0], acl[1]), acl[2]
	);
	return -1;

} // End of AddASA

static int AddASApblock(direction_t direction, char *arg) {

	if (strcasecmp(arg, "pblock") != 0) {
			yyprintf("Invalid port block: %s", arg);
			return -1;
	}

	int ret = -1;
	switch (direction) {
		case DIR_SRC:
		  ret = NewElement(EXgenericFlowID, OFFsrcPort, SIZEsrcPort, 1, CMP_EQ, FUNC_PBLOCK, NULLPtr);
		  break;
	  case DIR_DST:
		  ret = NewElement(EXgenericFlowID, OFFdstPort, SIZEdstPort, 1, CMP_EQ, FUNC_PBLOCK, NULLPtr);
		  break;
	  case DIR_UNSPEC:
		  ret = Connect_OR(
			  NewElement(EXgenericFlowID, OFFsrcPort, SIZEsrcPort, 1, CMP_EQ, FUNC_PBLOCK, NULLPtr),
			  NewElement(EXgenericFlowID, OFFdstPort, SIZEdstPort, 1, CMP_EQ, FUNC_PBLOCK, NULLPtr)
		  );
		  break;
		default:
			yyprintf("Invalid port direction");
	}

	return ret;
} // End of AddASApblock

static int AddNATString(char *event, char *natStr) {

	if (strcasecmp(event, "event") == 0) {
		int eventNum = natEventNum(natStr);
		if ( eventNum < 0 ) {
			yyprintf("Invalid NAT event type: %s", natStr);
			natEventInfo();
			return -1;
		}
		return NewElement(EXnatCommonID, OFFnatEvent, SIZEnatEvent, eventNum, CMP_EQ, FUNC_NONE, NULLPtr);
	} 

	yyprintf("Invalid NAT type: %s", event);

	return -1;

} // End of AddNATString

static int AddNAT(char *event, uint16_t comp, uint64_t number) {

	if (strcasecmp(event, "event") == 0) {
		if ( number > MAX_NAT_EVENTS ) {
			yyprintf("NAT event: %" PRIu64 " out of range\n", number);
			return -1;
		}
		return NewElement(EXnatCommonID, OFFnatEvent, SIZEnatEvent, number, comp, FUNC_NONE, NULLPtr);
	} 

	return -1;
} // End of AddNAT

static int AddNatPortBlocks(char *type, char *subtype, uint16_t comp, uint64_t number) {

	uint32_t offset = 0;
	if (strcasecmp(type, "pblock") == 0) {
		if (strcasecmp(subtype, "start") == 0) {
			offset = OFFnelblockStart;
		} else if (strcasecmp(subtype, "end") == 0) {
			offset = OFFnelblockEnd;
		} else if (strcasecmp(subtype, "step") == 0) {
			offset = OFFnelblockStep;
		} else if (strcasecmp(subtype, "size") == 0) {
			offset = OFFnelblockSize;
		} else {
			yyprintf("Unknown port block argument: %s\n", subtype);
			return -1;
		}
	} else {
			yyprintf("Unknown NAT argument: %s\n", type);
			return -1;
	}

	return NewElement(EXnatPortBlockID, offset, SIZEnelblockStart, number, comp, FUNC_NONE, NULLPtr);
	return -1;
} // End of AddNatPortBlocks

static int AddPayloadSSL(char *type, char *arg, char *opt) {
	if (strcasecmp(arg, "defined") == 0) {
		return NewElement(SSLindex, 0, 0, 0, CMP_EQ, FUNC_NONE, NULLPtr);
	} else if (strcasecmp(arg, "version") == 0) {
		if ( opt == NULL ){
			yyprintf("String %s is not a valid SSL/TLS version", arg);
			return -1;
		}
		unsigned int major, minor;
		if (sscanf(opt, "%1u.%1u", &major, &minor) != 2 || major > 3 || minor > 3 ) {
			yyprintf("String %s is not a valid SSL/TLS version", opt);
			return -1;
		}
		// if old SSL 1.0, 2.0 or 3.0
		if (major > 1 && minor > 0){
			yyprintf("String %s is not a valid SSL/TLS version", opt);
			return -1;
		}
		uint16_t version = 0;
		if ( strcasecmp(type, "tls") == 0 ) {
			if (major > 1){
				yyprintf("String %s is not a valid TLS version", opt);
				return -1;
			}
			// TLS
			version = (0x03 << 8) | (minor + 1);
		} else {
			if (minor > 0){
				yyprintf("String %s is not a valid SSL version", opt);
				return -1;
			}
			// SSL
			version = major << 8;
		}
		return NewElement(SSLindex, OFFsslVersion, SIZEsslVersion, version, CMP_EQ, FUNC_NONE, NULLPtr);
	} else if (strcasecmp(arg, "sni") == 0) {
		if ( opt == NULL || strlen(opt) > 64 ) {
			yyprintf("Invalid string %s for SSL/TLS sni name", opt != NULL ? opt : "");
			return -1;
		}
		data_t data = {.dataPtr=strdup(opt)};
		return NewElement(SSLindex, OFFsslSNI, SIZEsslSNI, 0, CMP_SUBSTRING, FUNC_NONE, data);
	}
	yyprintf("String %s is not a valid SSL/TLS filter", arg);
	return -1;
} // End of AddPayloadSSL

static int AddPayloadJA3(char *type, char *arg, char *opt) {
	if (strcasecmp(arg, "defined") == 0) {
		return NewElement(JA3index, OFFja3String, SIZEja3String, 0, CMP_EQ, FUNC_NONE, NULLPtr);
	} else if (IsMD5(arg) == 0) {
		yyprintf("String %s is not a valid ja3 string", arg);
		return -1;
	}
	data_t data = {.dataPtr=strdup(arg)};
	return NewElement(JA3index, OFFja3String, SIZEja3String, 0, CMP_STRING, FUNC_NONE, data);
} // End of AddPayloadJA3

static int AddPayloadJA4(char *type, char *arg, char *opt) {
	if (strcasecmp(arg, "defined") == 0) {
		return NewElement(JA4index, OFFja4String, SIZEja3String, 0, CMP_EQ, FUNC_NONE, NULLPtr);
	} else if ( ja4Check(arg) == 0 ){
		yyprintf("String %s is not a valid ja4 string", arg);
		return -1;
	}
	data_t data = {.dataPtr=strdup(arg)};
	return NewElement(JA4index, OFFja4String, SIZEja4String, 0, CMP_STRING, FUNC_NONE, data);
} // End of AddPayloadJA4

static int AddPayload(char *type, char *arg, char *opt) {

	if (strcasecmp(type, "content") == 0) {
		data_t data = {.dataPtr = arg};
		return NewElement(EXinPayloadID, 0, 0, 0, CMP_PAYLOAD, FUNC_NONE, data);
	} else if (strcasecmp(type, "regex") == 0) {
		int err[2];
		char *regexArg = opt ? opt : "";
		srx_Context *program = srx_CreateExt(arg, strlen(arg), regexArg, err, NULL, NULL);
		if ( !program ) {
			yyprintf("failed to compile regex: %s", arg);
			return -1;
		}
		data_t data = {.dataPtr = program};
		return NewElement(EXinPayloadID, 0, 0, 0, CMP_REGEX, FUNC_NONE, data);
	} else if (strcasecmp(type, "ssl") == 0 || strcasecmp(type, "tls") == 0) {
		return AddPayloadSSL(type, arg, opt);
	} else if (strcasecmp(type, "ja3") == 0) {
		return AddPayloadJA3(type, arg, opt);
	} else if (strcasecmp(type, "ja4") == 0) {
		return AddPayloadJA4(type, arg, opt);
	} else if (strcasecmp(type, "ja4s") == 0) {
#ifdef BUILDJA4
		if ( ja4sCheck(arg) == 0 ){
			yyprintf("String %s is not a valid ja4s string", arg);
			return -1;
		}
		data_t data = {.dataPtr=strdup(arg)};
		return NewElement(JA4index, OFFja4String, SIZEja4sString, 0, CMP_STRING, FUNC_NONE, data);
#else
		yyprintf("ja4s code not enabled");
		return -1;
#endif
	} else {
		yyprintf("Unknown PAYLOAD argument: %s\n", type);
		return -1;
	}

	return -1;
} // End of AddPayload

static int AddGeo(direction_t direction, char *geo) {

	geo += 4;
	if ( strlen(geo) != 2 ) {
			yyprintf("Unknown Geo country: %s. Need a two letter country code.", geo);
			return -1;
	}

	data_t data = {.dataVal = direction};
	int ret = -1;
	uint64_t geoVal = toupper(geo[0]) + (toupper(geo[1]) << 8);
	switch (direction) {
		case DIR_SRC:
			ret = NewElement(EXlocal, OFFgeoSrcIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, data);
			break;
		case DIR_DST:
			ret = NewElement(EXlocal, OFFgeoDstIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, data);
			break;
		case DIR_SRC_NAT:
			ret = NewElement(EXlocal, OFFgeoSrcNatIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, data);
			break;
		case DIR_DST_NAT:
			ret = NewElement(EXlocal, OFFgeoDstNatIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, data);
			break;
		case DIR_SRC_TUN:
			ret = NewElement(EXlocal, OFFgeoSrcTunIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, data);
			break;
		case DIR_DST_TUN:
			ret = NewElement(EXlocal, OFFgeoDstTunIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, data);
			break;
		case DIR_UNSPEC: {
			data_t srcData = {.dataVal = DIR_SRC};
			data_t dstData = {.dataVal = DIR_DST};
			ret = Connect_OR(
				NewElement(EXlocal, OFFgeoSrcIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, srcData),
				NewElement(EXlocal, OFFgeoDstIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, dstData)
			);
			} break;
		case DIR_UNSPEC_NAT: {
			data_t srcData = {.dataVal = DIR_SRC_NAT};
			data_t dstData = {.dataVal = DIR_DST_NAT};
			ret = Connect_OR(
				NewElement(EXlocal, OFFgeoSrcNatIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, srcData),
				NewElement(EXlocal, OFFgeoDstNatIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, dstData)
			);
			} break;
		case DIR_UNSPEC_TUN: {
			data_t srcData = {.dataVal = DIR_SRC_TUN};
			data_t dstData = {.dataVal = DIR_DST_TUN};
			ret = Connect_OR(
				NewElement(EXlocal, OFFgeoSrcTunIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, srcData),
				NewElement(EXlocal, OFFgeoDstTunIP, SizeGEOloc, geoVal, CMP_GEO, FUNC_NONE, dstData)
			);
			} break;
		default:
			yyprintf("Unknown Geo specifier");
	}

	return ret;
} // End of AddGeo

static int AddObservation(char *type, char *subType, uint16_t comp, uint64_t number) {

	if (strcasecmp(subType, "id") != 0) {
			yyprintf("Unknown observation specifier: %s", subType);
			return -1;
	}
	int ret = -1;
	if (strcasecmp(type, "domain") == 0) {
		ret =  NewElement(EXobservationID, OFFdomainID, SIZEdomainID, number, comp, FUNC_NONE, NULLPtr);
	} else if (strcasecmp(type, "point") == 0) {
		ret =  NewElement(EXobservationID, OFFpointID, SIZEpointID, number, comp, FUNC_NONE, NULLPtr);
	} else {
		yyprintf("Unknown observation specifier: %s", type);
	}

	return ret;
} // End of AddObservation

static int AddVRF(direction_t direction, uint16_t comp, uint64_t number) {

	int ret = -1;
	switch(direction) {
		case DIR_INGRESS:
			ret =  NewElement(EXvrfID, OFFingressVrf, SIZEingressVrf, number, comp, FUNC_NONE, NULLPtr);
			break;
		case DIR_EGRESS:
			ret =  NewElement(EXvrfID, OFFegressVrf, SIZEegressVrf, number, comp, FUNC_NONE, NULLPtr);
			break;
		default:
			yyprintf("Unknown vrf specifier");
	}

	return ret;
} // End of AddVRF

static int AddPFString(char *type, char *arg) {

	int ret = -1;
	if (strcasecmp(type, "action") == 0) {
		int pfAction = pfActionNr(arg);
		if ( pfAction < 0 ) {
				yyprintf("Invalid pf action: %s", arg);
				printf("Possible pf action values: ");
				pfListActions();
			} else {
				ret = NewElement(EXpfinfoID, OFFpfAction, SIZEpfAction, pfAction, CMP_EQ, FUNC_NONE, NULLPtr);
			}
	} else if (strcasecmp(type, "reason") == 0) {
		int pfReason = pfReasonNr(arg);
			if ( pfReason < 0 ) {
				yyprintf("Invalid pf reason: %s", arg);
				printf("Possible pf reason values: ");
				pfListReasons();
			} else {
				ret = NewElement(EXpfinfoID, OFFpfReason, SIZEpfReason, pfReason, CMP_EQ, FUNC_NONE, NULLPtr);
			}
	} else if (strcasecmp(type, "dir") == 0) {
		int pfDir = strcasecmp(arg, "in") == 0 ? 1: 0;
		ret = NewElement(EXpfinfoID, OFFpfDir, SIZEpfDir, pfDir, CMP_EQ, FUNC_NONE, NULLPtr);
	} else if (strcasecmp(type, "interface") == 0) {
		data_t data = {.dataPtr=strdup(arg)};
		ret = NewElement(EXpfinfoID, OFFpfIfName, SIZEpfIfName, 0, CMP_STRING, FUNC_NONE, data);
	} else {
		yyprintf("Invalid pf argument: %s", type);
	}
	return ret;
} // End of AddPFString

static int AddPFNumber(char *type, uint16_t comp, uint64_t number) {

	int ret = -1;
	if (strcasecmp(type, "rule") == 0) {
		ret = NewElement(EXpfinfoID, OFFpfRuleNr, SIZEpfRuleNr, number, comp, FUNC_NONE, NULLPtr);
	} else {
		yyprintf("Invalid pf argument: %s", type);
	}

	return ret;
} // End of AddPFNumber

static int AddIP(direction_t direction, char *IPstr) {

	int ret = -1;

	// if it's a tor node check
	if (strcasecmp(IPstr, "tor") == 0 ) {
		switch ( direction ) {
		case DIR_SRC:
			ret = Connect_OR(
				NewElement(EXipv4FlowID, OFFsrc4Addr, SIZEsrc4Addr, 1, CMP_EQ, FUNC_TOR_LOOKUP, NULLPtr), 
				NewElement(EXipv6FlowID, OFFsrc6Addr, SIZEsrc6Addr, 1, CMP_EQ, FUNC_TOR_LOOKUP, NULLPtr)
			);
			break;
		case DIR_DST:
			ret = Connect_OR(
				NewElement(EXipv4FlowID, OFFdst4Addr, SIZEdst4Addr, 1, CMP_EQ, FUNC_TOR_LOOKUP, NULLPtr), 
				NewElement(EXipv6FlowID, OFFdst6Addr, SIZEdst6Addr, 1, CMP_EQ, FUNC_TOR_LOOKUP, NULLPtr)
			);
			break;
		case DIR_UNSPEC: {
			int src = Connect_OR(
				NewElement(EXipv4FlowID, OFFsrc4Addr, SIZEsrc4Addr, 1, CMP_EQ, FUNC_TOR_LOOKUP, NULLPtr), 
				NewElement(EXipv6FlowID, OFFsrc6Addr, SIZEsrc6Addr, 1, CMP_EQ, FUNC_TOR_LOOKUP, NULLPtr)
			);
			int dst = Connect_OR(
				NewElement(EXipv4FlowID, OFFdst4Addr, SIZEdst4Addr, 1, CMP_EQ, FUNC_TOR_LOOKUP, NULLPtr), 
				NewElement(EXipv6FlowID, OFFdst6Addr, SIZEdst6Addr, 1, CMP_EQ, FUNC_TOR_LOOKUP, NULLPtr)
			);
			ret = Connect_OR(src,dst); 
			} break;
		default:
			yyprintf("Invalid direction for tor lookup");
		}
		return ret;
	} 

	// else normal IP compare
	int lookupMode = STRICT_IP;
	switch ( direction ) {
			case DIR_SRC:
			case DIR_DST:
			case DIR_UNSPEC:
				lookupMode = ALLOW_LOOKUP;
				break;
			default:
				lookupMode = STRICT_IP;
	} // End of switch

	int numIP = parseIP(IPstr, ipStack, lookupMode);
	if ( numIP <= 0)  {
		yyprintf("Can not parse/lookup %s to an IP address", IPstr);
		return -1;
	}

	switch ( direction ) {
		case DIR_SRC:
		case DIR_DST:
		case DIR_SRC_NAT:
		case DIR_DST_NAT:
		case DIR_SRC_TUN:
		case DIR_DST_TUN:
		case DIR_NEXT:
		case BGP_NEXT:
		case SRC_ROUTER:
			ret = ChainHosts(ipStack, numIP, direction);
			break;
		case DIR_UNSPEC: {
			uint32_t src = ChainHosts(ipStack, numIP, DIR_SRC);
			uint32_t dst = ChainHosts(ipStack, numIP, DIR_DST);
			ret = Connect_OR(src, dst);
			} break;
		case DIR_UNSPEC_NAT: {
			uint32_t src = ChainHosts(ipStack, numIP, DIR_SRC_NAT);
			uint32_t dst = ChainHosts(ipStack, numIP, DIR_DST_NAT);
			ret = Connect_OR(src, dst);
			} break;
		case DIR_UNSPEC_TUN: {
			uint32_t src = ChainHosts(ipStack, numIP, DIR_SRC_TUN);
			uint32_t dst = ChainHosts(ipStack, numIP, DIR_DST_TUN);
			ret = Connect_OR(src, dst);
			} break;
		default:
			yyprintf("Unknown direction for IP address");
	} // End of switch

	return ret;
} // End of AddIP

static int AddNet(direction_t direction, char *IPstr, char *maskStr) {
	
	int numIP = parseIP(IPstr, ipStack, STRICT_IP);
	if (numIP <= 0)  {
		yyprintf("Can not parse/lookup %s to an IP address", IPstr);
		return -1;
	}

	ipStack_t	mask;
	numIP = parseIP(maskStr, &mask, STRICT_IP);
	if (numIP <= 0)  {
		yyprintf("Can not parse %s as IP mask", maskStr);
		return -1;
	}

	if (ipStack[0].af != PF_INET || mask.af != PF_INET) {
		yyprintf("Net address %s and netmask: %s must be IPv4", IPstr, maskStr);
		return -1;
	}

	data_t data = {.dataVal = mask.ipaddr[1]};

	int ret = -1;
	switch ( direction ) {
		case DIR_SRC:
		case DIR_DST:
		case DIR_SRC_NAT:
		case DIR_DST_NAT:
		case DIR_SRC_TUN:
		case DIR_DST_TUN:
		case DIR_NEXT:
		case BGP_NEXT:
		case SRC_ROUTER:
			ret = NewIPElement(&ipStack[0], direction, CMP_NET, &data);
			break;
		case DIR_UNSPEC: {
			uint32_t src = NewIPElement(&ipStack[0], DIR_SRC, CMP_NET, &data);
			uint32_t dst = NewIPElement(&ipStack[0], DIR_DST, CMP_NET, &data);
			ret = Connect_OR(src, dst);
			} break;
		case DIR_UNSPEC_NAT: {
			uint32_t src = NewIPElement(&ipStack[0], DIR_SRC_NAT, CMP_NET, &data);
			uint32_t dst = NewIPElement(&ipStack[0], DIR_DST_NAT, CMP_NET, &data);
			ret = Connect_OR(src, dst);
			} break;
		case DIR_UNSPEC_TUN: {
			uint32_t src = NewIPElement(&ipStack[0], DIR_SRC_TUN, CMP_NET, &data);
			uint32_t dst = NewIPElement(&ipStack[0], DIR_DST_TUN, CMP_NET, &data);
			ret = Connect_OR(src, dst);
			} break;
		default:
			yyprintf("Unknown direction for IP address");
	} // End of switch

	return ret;
} // End of AddNet

static int AddNetPrefix(direction_t direction, char *IPstr, uint64_t prefix) {
	int numIP = parseIP(IPstr, ipStack, STRICT_IP);
	if (numIP <= 0)  {
		yyprintf("Can not parse/lookup %s to an IP address", IPstr);
		return -1;
	}

	data_t data[2];
	if (ipStack[0].af == PF_INET) {
		// IPv4 
		if (prefix >32 ) {
			yyprintf("Prefix %" PRIu64 " out of range for IPv4 address", prefix);
			return -1;
		}
		data[0].dataVal = 0xffffffffffffffffLL << (32 - prefix);
	} else {
		// IPv6
		if (prefix >128 ) {
			yyprintf("Prefix %" PRIu64 " out of range for IPv6 address", prefix);
			return -1;
		}
		if ( prefix > 64 ) {
			data[0].dataVal = 0xffffffffffffffffLL;
			data[1].dataVal = 0xffffffffffffffffLL << (128 - prefix);
		} else {
			data[0].dataVal = 0xffffffffffffffffLL << (64 - prefix);
			data[1].dataVal = 0;
		}
	}

	int ret = -1;
	switch (direction) {
		case DIR_SRC:
		case DIR_DST:
		case DIR_SRC_NAT:
		case DIR_DST_NAT:
		case DIR_SRC_TUN:
		case DIR_DST_TUN:
		case DIR_NEXT:
		case BGP_NEXT:
		case SRC_ROUTER:
			ret = NewIPElement(&ipStack[0], direction, CMP_NET, data);
			break;
		case DIR_UNSPEC: {
			uint32_t src = NewIPElement(&ipStack[0], DIR_SRC, CMP_NET, data);
			uint32_t dst = NewIPElement(&ipStack[0], DIR_DST, CMP_NET, data);
			ret = Connect_OR(src, dst);
			} break;
		case DIR_UNSPEC_NAT: {
			uint32_t src = NewIPElement(&ipStack[0], DIR_SRC_NAT, CMP_NET, data);
			uint32_t dst = NewIPElement(&ipStack[0], DIR_DST_NAT, CMP_NET, data);
			ret = Connect_OR(src, dst);
			} break;
		case DIR_UNSPEC_TUN: {
			uint32_t src = NewIPElement(&ipStack[0], DIR_SRC_TUN, CMP_NET, data);
			uint32_t dst = NewIPElement(&ipStack[0], DIR_DST_TUN, CMP_NET, data);
			ret = Connect_OR(src, dst);
			} break;
		default:
			yyprintf("Unknown direction for IP address");
	} // End of switch

	return ret;
} // End of AddNetPrefix

static int AddIPlist(direction_t direction, void *IPlist) {
	int ret = -1;
	data_t IPlistData = {IPlist};
	switch ( direction ) {
		case DIR_SRC:
			ret = Connect_OR(
				NewElement(EXipv4FlowID, OFFsrc4Addr, SIZEsrc4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXipv6FlowID, OFFsrc6Addr, SIZEsrc6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			break;
		case DIR_DST:
			ret = Connect_OR(
				NewElement(EXipv4FlowID, OFFdst4Addr, SIZEdst4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXipv6FlowID, OFFdst6Addr, SIZEdst6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			break;
		case DIR_SRC_NAT:
			ret = Connect_OR(
				NewElement(EXnatXlateIPv4ID, OFFxlateSrc4Addr, SIZExlateSrc4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXnatXlateIPv6ID, OFFxlateSrc6Addr, SIZExlateSrc6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			break;
		case DIR_DST_NAT:
			ret = Connect_OR(
				NewElement(EXnatXlateIPv4ID, OFFxlateDst4Addr, SIZExlateDst4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXnatXlateIPv6ID, OFFxlateDst6Addr, SIZExlateDst6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			break;
		case DIR_SRC_TUN:
			ret = Connect_OR(
				NewElement(EXtunIPv4ID, OFFtunSrc4Addr, SIZEtunSrc4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXtunIPv6ID, OFFtunSrc6Addr, SIZEtunSrc6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			break;
		case DIR_DST_TUN:
			ret = Connect_OR(
				NewElement(EXtunIPv4ID, OFFtunDst4Addr, SIZEtunDst4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXtunIPv6ID, OFFtunDst6Addr, SIZEtunDst6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			break;
		case DIR_NEXT:
			ret = Connect_OR(
				NewElement(EXipNextHopV4ID, OFFNextHopV4IP, SIZENextHopV4IP, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXipNextHopV6ID, OFFNextHopV6IP, SIZENextHopV6IP, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			break;
		case DIR_UNSPEC: {
			int v4 = Connect_OR(
				NewElement(EXipv4FlowID, OFFsrc4Addr, SIZEsrc4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXipv4FlowID, OFFdst4Addr, SIZEdst4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData) 
			);
			int v6 = Connect_OR(
				NewElement(EXipv6FlowID, OFFsrc6Addr, SIZEsrc6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData),
				NewElement(EXipv6FlowID, OFFdst6Addr, SIZEdst6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			ret = Connect_OR(v4, v6);
		} break;
		case DIR_UNSPEC_NAT: {
			int v4 = Connect_OR(
				NewElement(EXnatXlateIPv4ID, OFFxlateSrc4Addr, SIZExlateSrc4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXnatXlateIPv4ID, OFFxlateDst4Addr, SIZExlateDst4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData) 
			);
			int v6 = Connect_OR(
				NewElement(EXnatXlateIPv6ID, OFFxlateSrc6Addr, SIZExlateSrc6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData),
				NewElement(EXnatXlateIPv6ID, OFFxlateDst6Addr, SIZExlateDst6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			ret = Connect_OR(v4, v6);
		} break;
		case DIR_UNSPEC_TUN: {
			int v4 = Connect_OR(
				NewElement(EXtunIPv4ID, OFFtunSrc4Addr, SIZEtunSrc4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData), 
				NewElement(EXtunIPv4ID, OFFtunDst4Addr, SIZEtunDst4Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			int v6 = Connect_OR(
				NewElement(EXtunIPv6ID, OFFtunSrc6Addr, SIZEtunSrc6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData),
				NewElement(EXtunIPv6ID, OFFtunDst6Addr, SIZEtunDst6Addr, 0, CMP_IPLIST, FUNC_NONE, IPlistData)
			);
			ret = Connect_OR(v4, v6);
		} break;
		default:
			yyprintf("Unknown direction for IP list");
	}

	return ret;
} // AddIPlist

static struct IPListNode *mkNode(ipStack_t ipStack, int64_t prefix) {

	struct IPListNode *node = malloc(sizeof(struct IPListNode));
	if (node == NULL) {
		yyprintf("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
		return NULL;
	}

	node->ip[0] = ipStack.ipaddr[0];
	node->ip[1] = ipStack.ipaddr[1];
	node->mask[0] = 0xffffffffffffffffLL;
	node->mask[1] = 0xffffffffffffffffLL;

	if ( prefix > 0 ) {
		if (ipStack.af == PF_INET) {
		// IPv4 
			if (prefix >32 ) {
				yyprintf("Prefix %" PRIu64 " out of range for IPv4 address", prefix);
				return NULL;
			}
			node->mask[0] = 0;
			node->mask[1] = 0xffffffffffffffffLL << (32 - prefix);
		} else {
			// IPv6
			if (prefix >128 ) {
				yyprintf("Prefix %" PRIu64 " out of range for IPv6 address", prefix);
				return NULL;
			}
			if ( prefix > 64 ) {
				node->mask[0] = 0xffffffffffffffffLL;
				node->mask[1] = 0xffffffffffffffffLL << (128 - prefix);
			} else {
				node->mask[0] = 0xffffffffffffffffLL << (64 - prefix);
				node->mask[1] = 0;
			}
		}
	}
	return node;
}

static void *NewIplist(char *IPstr, int prefix) {
	IPlist_t *root = malloc(sizeof(IPlist_t));
	if (root == NULL) {
		yyprintf("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
		return NULL;
	}
	RB_INIT(root);

	int numIP = parseIP(IPstr, ipStack, ALLOW_LOOKUP);
	if ( numIP <= 0 ) {
		yyprintf("Can not parse/resolve %s to an IP address", IPstr);
		free(root);
		return NULL;
	}
		
	for (int i=0; i<numIP; i++ ) {
	  struct IPListNode *node = mkNode(ipStack[i], prefix);
		if ( node ) {
			RB_INSERT(IPtree, root, node);
		} else {
			free(root);
			return NULL;
		}
	}

	return root;
} // End of NewIPlist

static int InsertIPlist(void *IPlist, char *IPstr, int64_t prefix) {
	int numIP = parseIP(IPstr, ipStack, ALLOW_LOOKUP);
	if ( numIP <= 0 ) {
		// ret == - 2 means lookup failure
		yyprintf("Can not parse/resolve %s to an IP address", IPstr);
		return 0;
	}

	for (int i=0; i<numIP; i++ ) {
		struct IPListNode *node = mkNode(ipStack[i], prefix);
		if ( node ) {
			RB_INSERT(IPtree, (IPlist_t *)IPlist, node);
		} else {
			return 0;
		}
	}
	return 1;
} // End of InsertIPlist

static void *NewU64list(uint64_t num) {
	U64List_t *root = malloc(sizeof(U64List_t));
	if (root == NULL) {
		yyprintf("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
		return NULL;
	}
	RB_INIT(root);

  struct U64ListNode *node;
	if ((node = malloc(sizeof(struct U64ListNode))) == NULL) {
		yyprintf("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
		free(root);
		return NULL;
	}
	node->value = num;
	RB_INSERT(U64tree, root, node);

	return root;
} // End of NewU64list

static int InsertU64list(void *U64list, uint64_t num) {
	
	struct U64ListNode *node;
	if ((node = malloc(sizeof(struct U64ListNode))) == NULL) {
		yyprintf("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
		return 0;
	}
	node->value = num;
	RB_INSERT(U64tree, U64list, node);

	return 1;
} // End of InsertU64list

static int AddPortList(direction_t direction, void *U64List) {

	// check, that each element is a valid port number
	struct U64ListNode *node;
	RB_FOREACH(node, U64tree, (U64List_t *)U64List) {
		if ( node->value > 65535 ) {
			yyprintf("Port: %" PRIu64 " outside of range 0..65535", node->value);
			return -1;
		}
	}

	data_t U64ListPtr = {U64List};
	int ret = -1;
  switch ( direction ) {
	  case DIR_SRC:
		  ret = NewElement(EXgenericFlowID, OFFsrcPort, SIZEsrcPort, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr);
		  break;
	  case DIR_DST:
		  ret = NewElement(EXgenericFlowID, OFFdstPort, SIZEdstPort, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr);
		  break;
	  case DIR_UNSPEC:
		  ret = Connect_OR(
			  NewElement(EXgenericFlowID, OFFsrcPort, SIZEsrcPort, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr),
				NewElement(EXgenericFlowID, OFFdstPort, SIZEdstPort, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr)
		  );
		  break;
		case DIR_SRC_NAT:
		  ret = NewElement(EXnatXlatePortID, OFFxlateSrcPort, SIZExlateSrcPort, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr);
		  break;
	  case DIR_DST_NAT:
		  ret = NewElement(EXnatXlatePortID, OFFxlateDstPort, SIZExlateDstPort, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr);
		  break;
	  case DIR_UNSPEC_NAT:
		  ret = Connect_OR(
		  	NewElement(EXnatXlatePortID, OFFxlateSrcPort, SIZExlateSrcPort, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr),
		  	NewElement(EXnatXlatePortID, OFFxlateDstPort, SIZExlateDstPort, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr )
		  );
		  break;
	  default:
		  yyprintf("Unknown direction");
  } // End switch

	return ret;
} // AddPortList

static int AddASList(direction_t direction, void *U64List) {

	// check, that each element is a valid AS number
	struct U64ListNode *node;
	RB_FOREACH(node, U64tree, (U64List_t *)U64List) {
		if ( node->value > 0xFFFFFFFFLL ) {
			yyprintf("AS: %" PRIu64 " outside of range 32bit", node->value);
			return -1;
		}
	}

	data_t U64ListPtr = {U64List};
	int ret = -1;
  switch ( direction ) {
	  case DIR_SRC:
		  ret = NewElement(EXasRoutingID, OFFsrcAS, SIZEsrcAS, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr);
		  break;
	  case DIR_DST:
		  ret = NewElement(EXasRoutingID, OFFdstAS, SIZEdstAS, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr);
		  break;
	  case DIR_UNSPEC:
		  ret = Connect_OR(
			  NewElement(EXasRoutingID, OFFsrcAS, SIZEsrcAS, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr),
			  NewElement(EXasRoutingID, OFFdstAS, SIZEdstAS, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr)
		  );
		  break;
		case DIR_NEXT:
		  ret = NewElement(EXasAdjacentID, OFFnextAdjacentAS, SIZEnextAdjacentAS, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr);
			break;
		case DIR_PREV:
		  ret = NewElement(EXasAdjacentID, OFFprevAdjacentAS, SIZEprevAdjacentAS, 0, CMP_U64LIST, FUNC_NONE, U64ListPtr);
		  break;
	  default:
			yyprintf("Unknown direction");
  } // End of switch

	return ret;
} // AddASList

static int AddInterfaceNumber(direction_t direction, uint64_t num) {
	if ( num > 0xffffffffLL ) {
		yyprintf("Interface number out of range 0..2^32");
		return -1;
	}

	int ret = -1;
	switch ( direction ) {
		case DIR_UNSPEC:
			ret = Connect_OR(
				NewElement(EXflowMiscID, OFFinput, SIZEinput, num, CMP_EQ, FUNC_NONE, NULLPtr),
				NewElement(EXflowMiscID, OFFoutput, SIZEoutput, num, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			break;
		case DIR_IN: 
			ret = NewElement(EXflowMiscID, OFFinput, SIZEinput, num, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		case DIR_OUT: 
			ret = NewElement(EXflowMiscID, OFFoutput, SIZEoutput, num, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		default:
			yyprintf("Unknown interface direction");
	} // End of switch

	return ret;
} // End of AddInterfaceNumber

static int AddVlanNumber(direction_t direction, uint64_t num) {
	if ( num > 0xffffffffLL ) {
		yyprintf("Vlan number out of range 32bit");
		return -1;
	}

	int ret = -1;
	switch ( direction ) {
		case DIR_UNSPEC: {
			int src = Connect_OR(
			  NewElement(EXvLanID, OFFsrcVlan, SIZEsrcVlan, num, CMP_EQ, FUNC_NONE, NULLPtr),
			  NewElement(EXlayer2ID, OFFvlanID, SIZEvlanID, num, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			int dst = Connect_OR(
			  NewElement(EXvLanID, OFFdstVlan, SIZEdstVlan, num, CMP_EQ, FUNC_NONE, NULLPtr),
			  NewElement(EXlayer2ID, OFFpostVlanID, SIZEpostVlanID, num, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			ret = Connect_OR(src,dst);
			} break;
		case DIR_SRC: 
			ret = Connect_OR(
			  NewElement(EXvLanID, OFFsrcVlan, SIZEsrcVlan, num, CMP_EQ, FUNC_NONE, NULLPtr),
			  NewElement(EXlayer2ID, OFFvlanID, SIZEvlanID, num, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			break;
		case DIR_DST: 
			ret = Connect_OR(
			  NewElement(EXvLanID, OFFdstVlan, SIZEdstVlan, num, CMP_EQ, FUNC_NONE, NULLPtr),
			  NewElement(EXlayer2ID, OFFpostVlanID, SIZEpostVlanID, num, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			break;
		default:
			yyprintf("Unknown vlan direction");
	} // End of switch

	return ret;
} // End of AddVlanNumber

static int AddAsNumber(direction_t direction, uint16_t comp, uint64_t as) {
	if (as > UINT32_MAX ) {
		yyprintf("AS number of range");
		return -1;
  }

	int ret = -1;
  switch ( direction ) {
	  case DIR_SRC:
		  ret = NewElement(EXasRoutingID, OFFsrcAS, SIZEsrcAS, as, comp, FUNC_MMAS_LOOKUP, (data_t){.dataVal = OFFsrcAS});
		  break;
	  case DIR_DST:
		  ret = NewElement(EXasRoutingID, OFFdstAS, SIZEdstAS, as, comp, FUNC_MMAS_LOOKUP , (data_t){.dataVal = OFFdstAS});
		  break;
	  case DIR_UNSPEC:
		  ret = Connect_OR(
			  NewElement(EXasRoutingID, OFFsrcAS, SIZEsrcAS, as, comp, FUNC_MMAS_LOOKUP, (data_t){.dataVal = OFFsrcAS}),
			  NewElement(EXasRoutingID, OFFdstAS, SIZEdstAS, as, comp, FUNC_MMAS_LOOKUP ,(data_t){.dataVal = OFFdstAS} )
		  );
			break;
		case DIR_NEXT:
		  ret = NewElement(EXasAdjacentID, OFFnextAdjacentAS, SIZEnextAdjacentAS, as, comp, FUNC_MMAS_LOOKUP, NULLPtr);
			break;
		case DIR_PREV:
		  ret = NewElement(EXasAdjacentID, OFFprevAdjacentAS, SIZEprevAdjacentAS, as, comp, FUNC_MMAS_LOOKUP, NULLPtr);
		  break;
	  default:
			yyprintf("Unknown direction");
  } // End of switch

	return ret;
} // End of AddAsNumber

static int AddMaskNumber(direction_t direction, uint64_t num) {
	if ( num > 255 ) {
		yyprintf("Mask %" PRIu64 " out of range 0..255", num);
		return -1;
	}

	int ret = -1;
	switch ( direction ) {
		case DIR_UNSPEC:
			ret = Connect_OR(
				NewElement(EXflowMiscID, OFFsrcMask, SIZEsrcMask, num, CMP_EQ, FUNC_NONE, NULLPtr),
				NewElement(EXflowMiscID, OFFdstMask, SIZEdstMask, num, CMP_EQ, FUNC_NONE, NULLPtr)
			);
			break;
		case DIR_SRC: 
			ret = NewElement(EXflowMiscID, OFFsrcMask, SIZEsrcMask, num, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		case DIR_DST: 
			ret = NewElement(EXflowMiscID, OFFdstMask, SIZEdstMask, num, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		default:
			yyprintf("Invalid direction for mask");
	} // End of switch

	return ret;
} // End of AddMaskNumber

static int AddFlowDir(direction_t direction, int64_t dirNum) {

	int ret = -1;
	switch (direction) {
		case DIR_INGRESS:
	  	ret = NewElement(EXflowMiscID, OFFdir, SIZEdir, 0, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		case DIR_EGRESS:
	  	ret = NewElement(EXflowMiscID, OFFdir, SIZEdir, 1, CMP_EQ, FUNC_NONE, NULLPtr);
			break;
		case DIR_UNSPEC:
			if (dirNum != 0 && dirNum != 1) {
	 			yyprintf("Unknown flowdir: %" PRIi64, dirNum);
			} else {
	  		ret = NewElement(EXflowMiscID, OFFdir, SIZEdir, dirNum, CMP_EQ, FUNC_NONE, NULLPtr);
			}
			break;
		default:
	 			yyprintf("Unknown flowdir");
	}

	return ret;
} // End of AddFlowDirString
#line 2478 "filter/grammar.c"

/* For use in generated program */
#define yydepth (int)(yystack.s_mark - yystack.s_base)
#if YYBTYACC
#define yytrial (yyps->save)
#endif /* YYBTYACC */

#if YYDEBUG
#include <stdio.h>	/* needed for printf */
#endif

#include <stdlib.h>	/* needed for malloc, etc */
#include <string.h>	/* needed for memset */

/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(YYSTACKDATA *data)
{
    int i;
    unsigned newsize;
    YYINT *newss;
    YYSTYPE *newvs;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    YYLTYPE *newps;
#endif

    if ((newsize = data->stacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return YYENOMEM;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = (int) (data->s_mark - data->s_base);
    newss = (YYINT *)realloc(data->s_base, newsize * sizeof(*newss));
    if (newss == 0)
        return YYENOMEM;

    data->s_base = newss;
    data->s_mark = newss + i;

    newvs = (YYSTYPE *)realloc(data->l_base, newsize * sizeof(*newvs));
    if (newvs == 0)
        return YYENOMEM;

    data->l_base = newvs;
    data->l_mark = newvs + i;

#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    newps = (YYLTYPE *)realloc(data->p_base, newsize * sizeof(*newps));
    if (newps == 0)
        return YYENOMEM;

    data->p_base = newps;
    data->p_mark = newps + i;
#endif

    data->stacksize = newsize;
    data->s_last = data->s_base + newsize - 1;

#if YYDEBUG
    if (yydebug)
        fprintf(stderr, "%sdebug: stack size increased to %d\n", YYPREFIX, newsize);
#endif
    return 0;
}

#if YYPURE || defined(YY_NO_LEAKS)
static void yyfreestack(YYSTACKDATA *data)
{
    free(data->s_base);
    free(data->l_base);
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    free(data->p_base);
#endif
    memset(data, 0, sizeof(*data));
}
#else
#define yyfreestack(data) /* nothing */
#endif /* YYPURE || defined(YY_NO_LEAKS) */
#if YYBTYACC

static YYParseState *
yyNewState(unsigned size)
{
    YYParseState *p = (YYParseState *) malloc(sizeof(YYParseState));
    if (p == NULL) return NULL;

    p->yystack.stacksize = size;
    if (size == 0)
    {
        p->yystack.s_base = NULL;
        p->yystack.l_base = NULL;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        p->yystack.p_base = NULL;
#endif
        return p;
    }
    p->yystack.s_base    = (YYINT *) malloc(size * sizeof(YYINT));
    if (p->yystack.s_base == NULL) return NULL;
    p->yystack.l_base    = (YYSTYPE *) malloc(size * sizeof(YYSTYPE));
    if (p->yystack.l_base == NULL) return NULL;
    memset(p->yystack.l_base, 0, size * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    p->yystack.p_base    = (YYLTYPE *) malloc(size * sizeof(YYLTYPE));
    if (p->yystack.p_base == NULL) return NULL;
    memset(p->yystack.p_base, 0, size * sizeof(YYLTYPE));
#endif

    return p;
}

static void
yyFreeState(YYParseState *p)
{
    yyfreestack(&p->yystack);
    free(p);
}
#endif /* YYBTYACC */

#define YYABORT  goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR  goto yyerrlab
#if YYBTYACC
#define YYVALID        do { if (yyps->save)            goto yyvalid; } while(0)
#define YYVALID_NESTED do { if (yyps->save && \
                                yyps->save->save == 0) goto yyvalid; } while(0)
#endif /* YYBTYACC */

int
YYPARSE_DECL()
{
    int yym, yyn, yystate, yyresult;
#if YYBTYACC
    int yynewerrflag;
    YYParseState *yyerrctx = NULL;
#endif /* YYBTYACC */
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    YYLTYPE  yyerror_loc_range[3]; /* position of error start/end (0 unused) */
#endif
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
    if (yydebug)
        fprintf(stderr, "%sdebug[<# of symbols on state stack>]\n", YYPREFIX);
#endif
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    memset(yyerror_loc_range, 0, sizeof(yyerror_loc_range));
#endif

#if YYBTYACC
    yyps = yyNewState(0); if (yyps == 0) goto yyenomem;
    yyps->save = 0;
#endif /* YYBTYACC */
    yym = 0;
    /* yyn is set below */
    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;
    yystate = 0;

#if YYPURE
    memset(&yystack, 0, sizeof(yystack));
#endif

    if (yystack.s_base == NULL && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    yystack.s_mark = yystack.s_base;
    yystack.l_mark = yystack.l_base;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yystack.p_mark = yystack.p_base;
#endif
    yystate = 0;
    *yystack.s_mark = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
#if YYBTYACC
        do {
        if (yylvp < yylve)
        {
            /* we're currently re-reading tokens */
            yylval = *yylvp++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yylloc = *yylpp++;
#endif
            yychar = *yylexp++;
            break;
        }
        if (yyps->save)
        {
            /* in trial mode; save scanner results for future parse attempts */
            if (yylvp == yylvlim)
            {   /* Enlarge lexical value queue */
                size_t p = (size_t) (yylvp - yylvals);
                size_t s = (size_t) (yylvlim - yylvals);

                s += YYLVQUEUEGROWTH;
                if ((yylexemes = (YYINT *)realloc(yylexemes, s * sizeof(YYINT))) == NULL) goto yyenomem;
                if ((yylvals   = (YYSTYPE *)realloc(yylvals, s * sizeof(YYSTYPE))) == NULL) goto yyenomem;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                if ((yylpsns   = (YYLTYPE *)realloc(yylpsns, s * sizeof(YYLTYPE))) == NULL) goto yyenomem;
#endif
                yylvp   = yylve = yylvals + p;
                yylvlim = yylvals + s;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                yylpp   = yylpe = yylpsns + p;
                yylplim = yylpsns + s;
#endif
                yylexp  = yylexemes + p;
            }
            *yylexp = (YYINT) YYLEX;
            *yylvp++ = yylval;
            yylve++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            *yylpp++ = yylloc;
            yylpe++;
#endif
            yychar = *yylexp++;
            break;
        }
        /* normal operation, no conflict encountered */
#endif /* YYBTYACC */
        yychar = YYLEX;
#if YYBTYACC
        } while (0);
#endif /* YYBTYACC */
        if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            fprintf(stderr, "%s[%d]: state %d, reading token %d (%s)",
                            YYDEBUGSTR, yydepth, yystate, yychar, yys);
#ifdef YYSTYPE_TOSTRING
#if YYBTYACC
            if (!yytrial)
#endif /* YYBTYACC */
                fprintf(stderr, " <%s>", YYSTYPE_TOSTRING(yychar, yylval));
#endif
            fputc('\n', stderr);
        }
#endif
    }
#if YYBTYACC

    /* Do we have a conflict? */
    if (((yyn = yycindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
        yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
        YYINT ctry;

        if (yypath)
        {
            YYParseState *save;
#if YYDEBUG
            if (yydebug)
                fprintf(stderr, "%s[%d]: CONFLICT in state %d: following successful trial parse\n",
                                YYDEBUGSTR, yydepth, yystate);
#endif
            /* Switch to the next conflict context */
            save = yypath;
            yypath = save->save;
            save->save = NULL;
            ctry = save->ctry;
            if (save->state != yystate) YYABORT;
            yyFreeState(save);

        }
        else
        {

            /* Unresolved conflict - start/continue trial parse */
            YYParseState *save;
#if YYDEBUG
            if (yydebug)
            {
                fprintf(stderr, "%s[%d]: CONFLICT in state %d. ", YYDEBUGSTR, yydepth, yystate);
                if (yyps->save)
                    fputs("ALREADY in conflict, continuing trial parse.\n", stderr);
                else
                    fputs("Starting trial parse.\n", stderr);
            }
#endif
            save                  = yyNewState((unsigned)(yystack.s_mark - yystack.s_base + 1));
            if (save == NULL) goto yyenomem;
            save->save            = yyps->save;
            save->state           = yystate;
            save->errflag         = yyerrflag;
            save->yystack.s_mark  = save->yystack.s_base + (yystack.s_mark - yystack.s_base);
            memcpy (save->yystack.s_base, yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
            save->yystack.l_mark  = save->yystack.l_base + (yystack.l_mark - yystack.l_base);
            memcpy (save->yystack.l_base, yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            save->yystack.p_mark  = save->yystack.p_base + (yystack.p_mark - yystack.p_base);
            memcpy (save->yystack.p_base, yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
            ctry                  = yytable[yyn];
            if (yyctable[ctry] == -1)
            {
#if YYDEBUG
                if (yydebug && yychar >= YYEOF)
                    fprintf(stderr, "%s[%d]: backtracking 1 token\n", YYDEBUGSTR, yydepth);
#endif
                ctry++;
            }
            save->ctry = ctry;
            if (yyps->save == NULL)
            {
                /* If this is a first conflict in the stack, start saving lexemes */
                if (!yylexemes)
                {
                    yylexemes = (YYINT *) malloc((YYLVQUEUEGROWTH) * sizeof(YYINT));
                    if (yylexemes == NULL) goto yyenomem;
                    yylvals   = (YYSTYPE *) malloc((YYLVQUEUEGROWTH) * sizeof(YYSTYPE));
                    if (yylvals == NULL) goto yyenomem;
                    yylvlim   = yylvals + YYLVQUEUEGROWTH;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    yylpsns   = (YYLTYPE *) malloc((YYLVQUEUEGROWTH) * sizeof(YYLTYPE));
                    if (yylpsns == NULL) goto yyenomem;
                    yylplim   = yylpsns + YYLVQUEUEGROWTH;
#endif
                }
                if (yylvp == yylve)
                {
                    yylvp  = yylve = yylvals;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    yylpp  = yylpe = yylpsns;
#endif
                    yylexp = yylexemes;
                    if (yychar >= YYEOF)
                    {
                        *yylve++ = yylval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                        *yylpe++ = yylloc;
#endif
                        *yylexp  = (YYINT) yychar;
                        yychar   = YYEMPTY;
                    }
                }
            }
            if (yychar >= YYEOF)
            {
                yylvp--;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                yylpp--;
#endif
                yylexp--;
                yychar = YYEMPTY;
            }
            save->lexeme = (int) (yylvp - yylvals);
            yyps->save   = save;
        }
        if (yytable[yyn] == ctry)
        {
#if YYDEBUG
            if (yydebug)
                fprintf(stderr, "%s[%d]: state %d, shifting to state %d\n",
                                YYDEBUGSTR, yydepth, yystate, yyctable[ctry]);
#endif
            if (yychar < 0)
            {
                yylvp++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                yylpp++;
#endif
                yylexp++;
            }
            if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM)
                goto yyoverflow;
            yystate = yyctable[ctry];
            *++yystack.s_mark = (YYINT) yystate;
            *++yystack.l_mark = yylval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            *++yystack.p_mark = yylloc;
#endif
            yychar  = YYEMPTY;
            if (yyerrflag > 0) --yyerrflag;
            goto yyloop;
        }
        else
        {
            yyn = yyctable[ctry];
            goto yyreduce;
        }
    } /* End of code dealing with conflicts */
#endif /* YYBTYACC */
    if (((yyn = yysindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
#if YYDEBUG
        if (yydebug)
            fprintf(stderr, "%s[%d]: state %d, shifting to state %d\n",
                            YYDEBUGSTR, yydepth, yystate, yytable[yyn]);
#endif
        if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
        yystate = yytable[yyn];
        *++yystack.s_mark = yytable[yyn];
        *++yystack.l_mark = yylval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        *++yystack.p_mark = yylloc;
#endif
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if (((yyn = yyrindex[yystate]) != 0) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag != 0) goto yyinrecovery;
#if YYBTYACC

    yynewerrflag = 1;
    goto yyerrhandler;
    goto yyerrlab; /* redundant goto avoids 'unused label' warning */

yyerrlab:
    /* explicit YYERROR from an action -- pop the rhs of the rule reduced
     * before looking for error recovery */
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yystack.p_mark -= yym;
#endif

    yynewerrflag = 0;
yyerrhandler:
    while (yyps->save)
    {
        int ctry;
        YYParseState *save = yyps->save;
#if YYDEBUG
        if (yydebug)
            fprintf(stderr, "%s[%d]: ERROR in state %d, CONFLICT BACKTRACKING to state %d, %d tokens\n",
                            YYDEBUGSTR, yydepth, yystate, yyps->save->state,
                    (int)(yylvp - yylvals - yyps->save->lexeme));
#endif
        /* Memorize most forward-looking error state in case it's really an error. */
        if (yyerrctx == NULL || yyerrctx->lexeme < yylvp - yylvals)
        {
            /* Free old saved error context state */
            if (yyerrctx) yyFreeState(yyerrctx);
            /* Create and fill out new saved error context state */
            yyerrctx                 = yyNewState((unsigned)(yystack.s_mark - yystack.s_base + 1));
            if (yyerrctx == NULL) goto yyenomem;
            yyerrctx->save           = yyps->save;
            yyerrctx->state          = yystate;
            yyerrctx->errflag        = yyerrflag;
            yyerrctx->yystack.s_mark = yyerrctx->yystack.s_base + (yystack.s_mark - yystack.s_base);
            memcpy (yyerrctx->yystack.s_base, yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
            yyerrctx->yystack.l_mark = yyerrctx->yystack.l_base + (yystack.l_mark - yystack.l_base);
            memcpy (yyerrctx->yystack.l_base, yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yyerrctx->yystack.p_mark = yyerrctx->yystack.p_base + (yystack.p_mark - yystack.p_base);
            memcpy (yyerrctx->yystack.p_base, yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
            yyerrctx->lexeme         = (int) (yylvp - yylvals);
        }
        yylvp          = yylvals   + save->lexeme;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        yylpp          = yylpsns   + save->lexeme;
#endif
        yylexp         = yylexemes + save->lexeme;
        yychar         = YYEMPTY;
        yystack.s_mark = yystack.s_base + (save->yystack.s_mark - save->yystack.s_base);
        memcpy (yystack.s_base, save->yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
        yystack.l_mark = yystack.l_base + (save->yystack.l_mark - save->yystack.l_base);
        memcpy (yystack.l_base, save->yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        yystack.p_mark = yystack.p_base + (save->yystack.p_mark - save->yystack.p_base);
        memcpy (yystack.p_base, save->yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
        ctry           = ++save->ctry;
        yystate        = save->state;
        /* We tried shift, try reduce now */
        if ((yyn = yyctable[ctry]) >= 0) goto yyreduce;
        yyps->save     = save->save;
        save->save     = NULL;
        yyFreeState(save);

        /* Nothing left on the stack -- error */
        if (!yyps->save)
        {
#if YYDEBUG
            if (yydebug)
                fprintf(stderr, "%sdebug[%d,trial]: trial parse FAILED, entering ERROR mode\n",
                                YYPREFIX, yydepth);
#endif
            /* Restore state as it was in the most forward-advanced error */
            yylvp          = yylvals   + yyerrctx->lexeme;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yylpp          = yylpsns   + yyerrctx->lexeme;
#endif
            yylexp         = yylexemes + yyerrctx->lexeme;
            yychar         = yylexp[-1];
            yylval         = yylvp[-1];
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yylloc         = yylpp[-1];
#endif
            yystack.s_mark = yystack.s_base + (yyerrctx->yystack.s_mark - yyerrctx->yystack.s_base);
            memcpy (yystack.s_base, yyerrctx->yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
            yystack.l_mark = yystack.l_base + (yyerrctx->yystack.l_mark - yyerrctx->yystack.l_base);
            memcpy (yystack.l_base, yyerrctx->yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            yystack.p_mark = yystack.p_base + (yyerrctx->yystack.p_mark - yyerrctx->yystack.p_base);
            memcpy (yystack.p_base, yyerrctx->yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
            yystate        = yyerrctx->state;
            yyFreeState(yyerrctx);
            yyerrctx       = NULL;
        }
        yynewerrflag = 1;
    }
    if (yynewerrflag == 0) goto yyinrecovery;
#endif /* YYBTYACC */

    YYERROR_CALL("syntax error");
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yyerror_loc_range[1] = yylloc; /* lookahead position is error start position */
#endif

#if !YYBTYACC
    goto yyerrlab; /* redundant goto avoids 'unused label' warning */
yyerrlab:
#endif
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if (((yyn = yysindex[*yystack.s_mark]) != 0) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    fprintf(stderr, "%s[%d]: state %d, error recovery shifting to state %d\n",
                                    YYDEBUGSTR, yydepth, *yystack.s_mark, yytable[yyn]);
#endif
                if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
                yystate = yytable[yyn];
                *++yystack.s_mark = yytable[yyn];
                *++yystack.l_mark = yylval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                /* lookahead position is error end position */
                yyerror_loc_range[2] = yylloc;
                YYLLOC_DEFAULT(yyloc, yyerror_loc_range, 2); /* position of error span */
                *++yystack.p_mark = yyloc;
#endif
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    fprintf(stderr, "%s[%d]: error recovery discarding state %d\n",
                                    YYDEBUGSTR, yydepth, *yystack.s_mark);
#endif
                if (yystack.s_mark <= yystack.s_base) goto yyabort;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                /* the current TOS position is the error start position */
                yyerror_loc_range[1] = *yystack.p_mark;
#endif
#if defined(YYDESTRUCT_CALL)
#if YYBTYACC
                if (!yytrial)
#endif /* YYBTYACC */
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    YYDESTRUCT_CALL("error: discarding state",
                                    yystos[*yystack.s_mark], yystack.l_mark, yystack.p_mark);
#else
                    YYDESTRUCT_CALL("error: discarding state",
                                    yystos[*yystack.s_mark], yystack.l_mark);
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */
#endif /* defined(YYDESTRUCT_CALL) */
                --yystack.s_mark;
                --yystack.l_mark;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                --yystack.p_mark;
#endif
            }
        }
    }
    else
    {
        if (yychar == YYEOF) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
            fprintf(stderr, "%s[%d]: state %d, error recovery discarding token %d (%s)\n",
                            YYDEBUGSTR, yydepth, yystate, yychar, yys);
        }
#endif
#if defined(YYDESTRUCT_CALL)
#if YYBTYACC
        if (!yytrial)
#endif /* YYBTYACC */
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
            YYDESTRUCT_CALL("error: discarding token", yychar, &yylval, &yylloc);
#else
            YYDESTRUCT_CALL("error: discarding token", yychar, &yylval);
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */
#endif /* defined(YYDESTRUCT_CALL) */
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
    yym = yylen[yyn];
#if YYDEBUG
    if (yydebug)
    {
        fprintf(stderr, "%s[%d]: state %d, reducing by rule %d (%s)",
                        YYDEBUGSTR, yydepth, yystate, yyn, yyrule[yyn]);
#ifdef YYSTYPE_TOSTRING
#if YYBTYACC
        if (!yytrial)
#endif /* YYBTYACC */
            if (yym > 0)
            {
                int i;
                fputc('<', stderr);
                for (i = yym; i > 0; i--)
                {
                    if (i != yym) fputs(", ", stderr);
                    fputs(YYSTYPE_TOSTRING(yystos[yystack.s_mark[1-i]],
                                           yystack.l_mark[1-i]), stderr);
                }
                fputc('>', stderr);
            }
#endif
        fputc('\n', stderr);
    }
#endif
    if (yym > 0)
        yyval = yystack.l_mark[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)

    /* Perform position reduction */
    memset(&yyloc, 0, sizeof(yyloc));
#if YYBTYACC
    if (!yytrial)
#endif /* YYBTYACC */
    {
        YYLLOC_DEFAULT(yyloc, &yystack.p_mark[-yym], yym);
        /* just in case YYERROR is invoked within the action, save
           the start of the rhs as the error start position */
        yyerror_loc_range[1] = yystack.p_mark[1-yym];
    }
#endif

    switch (yyn)
    {
case 2:
#line 201 "filter/grammar.y"
	{   
		StartNode = yystack.l_mark[0].value; 
	}
#line 3153 "filter/grammar.c"
break;
case 3:
#line 206 "filter/grammar.y"
	{ /* this is an unconditionally true expression, as a filter applies in any case */
		data_t data = {.dataVal=1};
		yyval.param.self = NewElement(EXnull, 0, 0, 0, CMP_EQ, FUNC_NONE, data);
	}
#line 3161 "filter/grammar.c"
break;
case 4:
#line 211 "filter/grammar.y"
	{ 
		yyval.param.self = NewElement(EXipv4FlowID, OFFsrc4Addr, 0, 0, CMP_EQ, FUNC_NONE, NULLPtr); 
	}
#line 3168 "filter/grammar.c"
break;
case 5:
#line 215 "filter/grammar.y"
	{ 
		yyval.param.self = NewElement(EXipv6FlowID, OFFsrc6Addr, 0, 0, CMP_EQ, FUNC_NONE, NULLPtr); 
	}
#line 3175 "filter/grammar.c"
break;
case 6:
#line 219 "filter/grammar.y"
	{
	  yyval.param.self  = AddIdent(yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3182 "filter/grammar.c"
break;
case 7:
#line 223 "filter/grammar.y"
	{
		yyval.param.self = NewElement(EXlocal, OFFflowCount, SIZEflowCount, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULLPtr); 
	}
#line 3189 "filter/grammar.c"
break;
case 8:
#line 227 "filter/grammar.y"
	{
	  yyval.param.self  = AddEngineNum("type", yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
  }
#line 3196 "filter/grammar.c"
break;
case 9:
#line 231 "filter/grammar.y"
	{
	  yyval.param.self  = AddEngineNum("id", yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3203 "filter/grammar.c"
break;
case 10:
#line 235 "filter/grammar.y"
	{
		yyval.param.self  = AddEngineNum(yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3210 "filter/grammar.c"
break;
case 11:
#line 239 "filter/grammar.y"
	{
 	  yyval.param.self  = AddExporterNum(yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3217 "filter/grammar.c"
break;
case 12:
#line 243 "filter/grammar.y"
	{ 
    yyval.param.self = AddProto(yystack.l_mark[-2].param.direction, NULL, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
  }
#line 3224 "filter/grammar.c"
break;
case 13:
#line 247 "filter/grammar.y"
	{
    yyval.param.self = AddProto(yystack.l_mark[-2].param.direction, yystack.l_mark[0].s, 0); if ( yyval.param.self < 0 ) YYABORT;
  }
#line 3231 "filter/grammar.c"
break;
case 14:
#line 251 "filter/grammar.y"
	{
    yyval.param.self = AddProto(yystack.l_mark[-2].param.direction, "icmp", 0); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3238 "filter/grammar.c"
break;
case 15:
#line 255 "filter/grammar.y"
	{
		yyval.param.self = AddPortNumber(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3245 "filter/grammar.c"
break;
case 16:
#line 259 "filter/grammar.y"
	{
		yyval.param.self = AddICMP("type", yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3252 "filter/grammar.c"
break;
case 17:
#line 263 "filter/grammar.y"
	{
		yyval.param.self = AddICMP("code", yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3259 "filter/grammar.c"
break;
case 18:
#line 267 "filter/grammar.y"
	{
		yyval.param.self  = AddICMP(yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3266 "filter/grammar.c"
break;
case 19:
#line 271 "filter/grammar.y"
	{
		yyval.param.self = AddFlagsNumber(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3273 "filter/grammar.c"
break;
case 20:
#line 275 "filter/grammar.y"
	{
		yyval.param.self = AddFlagsString(yystack.l_mark[-2].param.direction, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3280 "filter/grammar.c"
break;
case 21:
#line 279 "filter/grammar.y"
	{
	  yyval.param.self = AddTosNumber(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3287 "filter/grammar.c"
break;
case 22:
#line 283 "filter/grammar.y"
	{
	  yyval.param.self = AddIPttl(yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3294 "filter/grammar.c"
break;
case 23:
#line 287 "filter/grammar.y"
	{
	  yyval.param.self = AddFwdStatNum(yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3301 "filter/grammar.c"
break;
case 24:
#line 291 "filter/grammar.y"
	{
	  yyval.param.self = AddFwdStatString(yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
  }
#line 3308 "filter/grammar.c"
break;
case 25:
#line 295 "filter/grammar.y"
	{
		yyval.param.self = NewElement(EXgenericFlowID, 0, SIZEmsecLast, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_DURATION, NULLPtr); 
	}
#line 3315 "filter/grammar.c"
break;
case 26:
#line 299 "filter/grammar.y"
	{
		yyval.param.self = NewElement(EXgenericFlowID, 0, SIZEmsecLast, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_PPS, NULLPtr);
	}
#line 3322 "filter/grammar.c"
break;
case 27:
#line 303 "filter/grammar.y"
	{
		yyval.param.self = NewElement(EXgenericFlowID, 0, SIZEmsecLast, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_BPS, NULLPtr);
	}
#line 3329 "filter/grammar.c"
break;
case 28:
#line 307 "filter/grammar.y"
	{
		yyval.param.self = NewElement(EXgenericFlowID, 0, SIZEmsecLast, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_BPP, NULLPtr); 
	}
#line 3336 "filter/grammar.c"
break;
case 29:
#line 311 "filter/grammar.y"
	{
		yyval.param.self = AddPackets(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3343 "filter/grammar.c"
break;
case 30:
#line 315 "filter/grammar.y"
	{
		yyval.param.self = NewElement(EXcntFlowID, OFFflows, SIZEflows, yystack.l_mark[0].value, yystack.l_mark[-1].param.comp, FUNC_NONE, NULLPtr); 
	}
#line 3350 "filter/grammar.c"
break;
case 31:
#line 319 "filter/grammar.y"
	{
		yyval.param.self = AddBytes(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3357 "filter/grammar.c"
break;
case 32:
#line 323 "filter/grammar.y"
	{ 	
		yyval.param.self = AddIP(yystack.l_mark[-2].param.direction, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3364 "filter/grammar.c"
break;
case 33:
#line 327 "filter/grammar.y"
	{
		yyval.param.self = AddNet(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].s, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3371 "filter/grammar.c"
break;
case 34:
#line 331 "filter/grammar.y"
	{
		yyval.param.self = AddNetPrefix(yystack.l_mark[-4].param.direction, yystack.l_mark[-2].s, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3378 "filter/grammar.c"
break;
case 35:
#line 335 "filter/grammar.y"
	{
		yyval.param.self = AddInterfaceNumber(yystack.l_mark[-2].param.direction, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3385 "filter/grammar.c"
break;
case 36:
#line 339 "filter/grammar.y"
	{
		yyval.param.self = AddVlanNumber(yystack.l_mark[-2].param.direction, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3392 "filter/grammar.c"
break;
case 37:
#line 343 "filter/grammar.y"
	{
		yyval.param.self = AddAsNumber(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3399 "filter/grammar.c"
break;
case 38:
#line 347 "filter/grammar.y"
	{
		yyval.param.self = AddMaskNumber(yystack.l_mark[-2].param.direction, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3406 "filter/grammar.c"
break;
case 39:
#line 351 "filter/grammar.y"
	{
		yyval.param.self = AddEthertype(yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3413 "filter/grammar.c"
break;
case 40:
#line 355 "filter/grammar.y"
	{
		yyval.param.self = AddFlowDir(DIR_UNSPEC, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3420 "filter/grammar.c"
break;
case 41:
#line 359 "filter/grammar.y"
	{
		yyval.param.self = AddFlowDir(yystack.l_mark[0].param.direction, -1); if ( yyval.param.self < 0 ) YYABORT;
  }
#line 3427 "filter/grammar.c"
break;
case 42:
#line 363 "filter/grammar.y"
	{	
		yyval.param.self = AddMPLS(yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3434 "filter/grammar.c"
break;
case 43:
#line 367 "filter/grammar.y"
	{	
		yyval.param.self = AddMPLS("any", yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3441 "filter/grammar.c"
break;
case 44:
#line 371 "filter/grammar.y"
	{	
		yyval.param.self = AddMAC(yystack.l_mark[-2].param.direction, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3448 "filter/grammar.c"
break;
case 45:
#line 375 "filter/grammar.y"
	{
		yyval.param.self = AddLatency(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3455 "filter/grammar.c"
break;
case 46:
#line 379 "filter/grammar.y"
	{
		yyval.param.self = AddASAString(yystack.l_mark[-1].s, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3462 "filter/grammar.c"
break;
case 47:
#line 383 "filter/grammar.y"
	{
		switch (yystack.l_mark[0].param.direction) {
			case DIR_INGRESS:
				yyval.param.self = AddASAString(yystack.l_mark[-1].s, "ingress");
				break;
			case DIR_EGRESS:
				yyval.param.self = AddASAString(yystack.l_mark[-1].s, "egress");
				break;
			default:
				yyval.param.self = -1;
				    yyerror("Unknown direction specifier");
		}
		if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3480 "filter/grammar.c"
break;
case 48:
#line 398 "filter/grammar.y"
	{
		yyval.param.self = AddASA(yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3487 "filter/grammar.c"
break;
case 49:
#line 402 "filter/grammar.y"
	{
		yyval.param.self = AddASApblock(yystack.l_mark[-4].param.direction, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3494 "filter/grammar.c"
break;
case 50:
#line 406 "filter/grammar.y"
	{
		yyval.param.self = AddACL(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3501 "filter/grammar.c"
break;
case 51:
#line 410 "filter/grammar.y"
	{
		yyval.param.self = AddNATString(yystack.l_mark[-1].s, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3508 "filter/grammar.c"
break;
case 52:
#line 414 "filter/grammar.y"
	{
		yyval.param.self = AddNatPortBlocks(yystack.l_mark[-3].s, yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3515 "filter/grammar.c"
break;
case 53:
#line 418 "filter/grammar.y"
	{
		yyval.param.self = AddNAT(yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT; 
	}
#line 3522 "filter/grammar.c"
break;
case 54:
#line 422 "filter/grammar.y"
	{
		yyval.param.self = AddPayload(yystack.l_mark[-1].s, yystack.l_mark[0].s, NULL); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3529 "filter/grammar.c"
break;
case 55:
#line 426 "filter/grammar.y"
	{
		yyval.param.self = AddPayload(yystack.l_mark[-2].s, yystack.l_mark[-1].s, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3536 "filter/grammar.c"
break;
case 56:
#line 430 "filter/grammar.y"
	{
		yyval.param.self = AddGeo(yystack.l_mark[-1].param.direction, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3543 "filter/grammar.c"
break;
case 57:
#line 434 "filter/grammar.y"
	{
		yyval.param.self = AddObservation(yystack.l_mark[-3].s, yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3550 "filter/grammar.c"
break;
case 58:
#line 438 "filter/grammar.y"
	{
		yyval.param.self = AddVRF(yystack.l_mark[-3].param.direction, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3557 "filter/grammar.c"
break;
case 59:
#line 442 "filter/grammar.y"
	{
		yyval.param.self = AddPFString(yystack.l_mark[-1].s, yystack.l_mark[0].s); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3564 "filter/grammar.c"
break;
case 60:
#line 446 "filter/grammar.y"
	{
		yyval.param.self = AddPFNumber(yystack.l_mark[-2].s, yystack.l_mark[-1].param.comp, yystack.l_mark[0].value); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3571 "filter/grammar.c"
break;
case 61:
#line 450 "filter/grammar.y"
	{
		switch (yystack.l_mark[0].param.direction) {
			case DIR_IN:
				yyval.param.self = AddPFString(yystack.l_mark[-1].s, "in");
				break;
			case DIR_OUT:
				yyval.param.self = AddPFString(yystack.l_mark[-1].s, "out");
				break;
			case DIR_UNSPEC_NAT:
				yyval.param.self = AddPFString(yystack.l_mark[-1].s, "nat");
				break;
			default:
				yyval.param.self = -1;
				    yyerror("Unknown direction specifier");
		}
		if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3592 "filter/grammar.c"
break;
case 62:
#line 468 "filter/grammar.y"
	{ 	
		yyval.param.self = AddIPlist(yystack.l_mark[-5].param.direction, yystack.l_mark[-1].list); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3599 "filter/grammar.c"
break;
case 63:
#line 472 "filter/grammar.y"
	{
		yyval.param.self = AddPortList(yystack.l_mark[-5].param.direction, yystack.l_mark[-1].list); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3606 "filter/grammar.c"
break;
case 64:
#line 476 "filter/grammar.y"
	{
		yyval.param.self = AddASList(yystack.l_mark[-5].param.direction, yystack.l_mark[-1].list); if ( yyval.param.self < 0 ) YYABORT;
	}
#line 3613 "filter/grammar.c"
break;
case 65:
#line 482 "filter/grammar.y"
	{ 
		yyval.list = NewIplist(yystack.l_mark[0].s, -1); if ( yyval.list == NULL ) YYABORT;
	}
#line 3620 "filter/grammar.c"
break;
case 66:
#line 486 "filter/grammar.y"
	{ 
		yyval.list = NewIplist(yystack.l_mark[-2].s, yystack.l_mark[0].value); if ( yyval.list == NULL ) YYABORT;
	}
#line 3627 "filter/grammar.c"
break;
case 67:
#line 490 "filter/grammar.y"
	{ 
		if (InsertIPlist(yystack.l_mark[-1].list, yystack.l_mark[0].s, -1) == 0 ) YYABORT;
	}
#line 3634 "filter/grammar.c"
break;
case 68:
#line 494 "filter/grammar.y"
	{ 
		if (InsertIPlist(yystack.l_mark[-2].list, yystack.l_mark[0].s, -1) == 0 ) YYABORT;
	}
#line 3641 "filter/grammar.c"
break;
case 69:
#line 498 "filter/grammar.y"
	{ 
		if (InsertIPlist(yystack.l_mark[-3].list, yystack.l_mark[-2].s, yystack.l_mark[0].value) == 0 ) YYABORT;
	}
#line 3648 "filter/grammar.c"
break;
case 70:
#line 503 "filter/grammar.y"
	{ 
		yyval.list = NewU64list(yystack.l_mark[0].value); if ( yyval.list == NULL ) YYABORT;
	}
#line 3655 "filter/grammar.c"
break;
case 71:
#line 507 "filter/grammar.y"
	{ 
		if (InsertU64list(yystack.l_mark[-1].list, yystack.l_mark[0].value) == 0 ) YYABORT;
	}
#line 3662 "filter/grammar.c"
break;
case 72:
#line 511 "filter/grammar.y"
	{ 
		if (InsertU64list(yystack.l_mark[-2].list, yystack.l_mark[0].value) == 0 ) YYABORT;
	}
#line 3669 "filter/grammar.c"
break;
case 73:
#line 517 "filter/grammar.y"
	{ yyval.param.comp = CMP_EQ; }
#line 3674 "filter/grammar.c"
break;
case 74:
#line 518 "filter/grammar.y"
	{ yyval.param.comp = CMP_EQ; }
#line 3679 "filter/grammar.c"
break;
case 75:
#line 519 "filter/grammar.y"
	{ yyval.param.comp = CMP_LT; }
#line 3684 "filter/grammar.c"
break;
case 76:
#line 520 "filter/grammar.y"
	{ yyval.param.comp = CMP_GT; }
#line 3689 "filter/grammar.c"
break;
case 77:
#line 521 "filter/grammar.y"
	{ yyval.param.comp = CMP_LE; }
#line 3694 "filter/grammar.c"
break;
case 78:
#line 522 "filter/grammar.y"
	{ yyval.param.comp = CMP_GE; }
#line 3699 "filter/grammar.c"
break;
case 79:
#line 526 "filter/grammar.y"
	{ yyval.param.direction = DIR_UNSPEC;     }
#line 3704 "filter/grammar.c"
break;
case 80:
#line 527 "filter/grammar.y"
	{ yyval.param.direction = DIR_SRC;        }
#line 3709 "filter/grammar.c"
break;
case 81:
#line 528 "filter/grammar.y"
	{ yyval.param.direction = DIR_DST;        }
#line 3714 "filter/grammar.c"
break;
case 82:
#line 529 "filter/grammar.y"
	{ yyval.param.direction = DIR_SRC_NAT;	  }
#line 3719 "filter/grammar.c"
break;
case 83:
#line 530 "filter/grammar.y"
	{ yyval.param.direction = DIR_DST_NAT;	  }
#line 3724 "filter/grammar.c"
break;
case 84:
#line 531 "filter/grammar.y"
	{ yyval.param.direction = DIR_SRC_TUN;    }
#line 3729 "filter/grammar.c"
break;
case 85:
#line 532 "filter/grammar.y"
	{ yyval.param.direction = DIR_DST_TUN;    }
#line 3734 "filter/grammar.c"
break;
case 86:
#line 533 "filter/grammar.y"
	{ yyval.param.direction = DIR_UNSPEC_NAT; }
#line 3739 "filter/grammar.c"
break;
case 87:
#line 534 "filter/grammar.y"
	{ yyval.param.direction = DIR_UNSPEC_TUN; }
#line 3744 "filter/grammar.c"
break;
case 88:
#line 535 "filter/grammar.y"
	{ yyval.param.direction = DIR_IN;         }
#line 3749 "filter/grammar.c"
break;
case 89:
#line 536 "filter/grammar.y"
	{ yyval.param.direction = DIR_OUT;        }
#line 3754 "filter/grammar.c"
break;
case 90:
#line 537 "filter/grammar.y"
	{ yyval.param.direction = DIR_IN_SRC;     }
#line 3759 "filter/grammar.c"
break;
case 91:
#line 538 "filter/grammar.y"
	{ yyval.param.direction = DIR_IN_DST;     }
#line 3764 "filter/grammar.c"
break;
case 92:
#line 539 "filter/grammar.y"
	{ yyval.param.direction = DIR_OUT_SRC;	  }
#line 3769 "filter/grammar.c"
break;
case 93:
#line 540 "filter/grammar.y"
	{ yyval.param.direction = DIR_OUT_DST;    }
#line 3774 "filter/grammar.c"
break;
case 94:
#line 541 "filter/grammar.y"
	{ yyval.param.direction = DIR_INGRESS;    }
#line 3779 "filter/grammar.c"
break;
case 95:
#line 542 "filter/grammar.y"
	{ yyval.param.direction = DIR_EGRESS;     }
#line 3784 "filter/grammar.c"
break;
case 96:
#line 543 "filter/grammar.y"
	{ yyval.param.direction = DIR_CLIENT;     }
#line 3789 "filter/grammar.c"
break;
case 97:
#line 544 "filter/grammar.y"
	{ yyval.param.direction = DIR_SERVER;     }
#line 3794 "filter/grammar.c"
break;
case 98:
#line 545 "filter/grammar.y"
	{ yyval.param.direction = DIR_PREV;       }
#line 3799 "filter/grammar.c"
break;
case 99:
#line 546 "filter/grammar.y"
	{ yyval.param.direction = DIR_NEXT;       }
#line 3804 "filter/grammar.c"
break;
case 100:
#line 547 "filter/grammar.y"
	{ yyval.param.direction = BGP_NEXT;	      }
#line 3809 "filter/grammar.c"
break;
case 101:
#line 548 "filter/grammar.y"
	{ yyval.param.direction = SRC_ROUTER;     }
#line 3814 "filter/grammar.c"
break;
case 102:
#line 549 "filter/grammar.y"
	{ yyval.param.direction = SRC_ROUTER;     }
#line 3819 "filter/grammar.c"
break;
case 103:
#line 552 "filter/grammar.y"
	{ yyval.value = yystack.l_mark[0].param.self; }
#line 3824 "filter/grammar.c"
break;
case 104:
#line 553 "filter/grammar.y"
	{ yyval.value = Connect_OR(yystack.l_mark[-2].value, yystack.l_mark[0].value);   }
#line 3829 "filter/grammar.c"
break;
case 105:
#line 554 "filter/grammar.y"
	{ yyval.value = Connect_AND(yystack.l_mark[-2].value, yystack.l_mark[0].value);  }
#line 3834 "filter/grammar.c"
break;
case 106:
#line 555 "filter/grammar.y"
	{ yyval.value = Invert(yystack.l_mark[0].value); }
#line 3839 "filter/grammar.c"
break;
case 107:
#line 556 "filter/grammar.y"
	{ yyval.value = yystack.l_mark[-1].value; }
#line 3844 "filter/grammar.c"
break;
#line 3846 "filter/grammar.c"
    default:
        break;
    }
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yystack.p_mark -= yym;
#endif
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
        {
            fprintf(stderr, "%s[%d]: after reduction, ", YYDEBUGSTR, yydepth);
#ifdef YYSTYPE_TOSTRING
#if YYBTYACC
            if (!yytrial)
#endif /* YYBTYACC */
                fprintf(stderr, "result is <%s>, ", YYSTYPE_TOSTRING(yystos[YYFINAL], yyval));
#endif
            fprintf(stderr, "shifting from state 0 to final state %d\n", YYFINAL);
        }
#endif
        yystate = YYFINAL;
        *++yystack.s_mark = YYFINAL;
        *++yystack.l_mark = yyval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        *++yystack.p_mark = yyloc;
#endif
        if (yychar < 0)
        {
#if YYBTYACC
            do {
            if (yylvp < yylve)
            {
                /* we're currently re-reading tokens */
                yylval = *yylvp++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                yylloc = *yylpp++;
#endif
                yychar = *yylexp++;
                break;
            }
            if (yyps->save)
            {
                /* in trial mode; save scanner results for future parse attempts */
                if (yylvp == yylvlim)
                {   /* Enlarge lexical value queue */
                    size_t p = (size_t) (yylvp - yylvals);
                    size_t s = (size_t) (yylvlim - yylvals);

                    s += YYLVQUEUEGROWTH;
                    if ((yylexemes = (YYINT *)realloc(yylexemes, s * sizeof(YYINT))) == NULL)
                        goto yyenomem;
                    if ((yylvals   = (YYSTYPE *)realloc(yylvals, s * sizeof(YYSTYPE))) == NULL)
                        goto yyenomem;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    if ((yylpsns   = (YYLTYPE *)realloc(yylpsns, s * sizeof(YYLTYPE))) == NULL)
                        goto yyenomem;
#endif
                    yylvp   = yylve = yylvals + p;
                    yylvlim = yylvals + s;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                    yylpp   = yylpe = yylpsns + p;
                    yylplim = yylpsns + s;
#endif
                    yylexp  = yylexemes + p;
                }
                *yylexp = (YYINT) YYLEX;
                *yylvp++ = yylval;
                yylve++;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
                *yylpp++ = yylloc;
                yylpe++;
#endif
                yychar = *yylexp++;
                break;
            }
            /* normal operation, no conflict encountered */
#endif /* YYBTYACC */
            yychar = YYLEX;
#if YYBTYACC
            } while (0);
#endif /* YYBTYACC */
            if (yychar < 0) yychar = YYEOF;
#if YYDEBUG
            if (yydebug)
            {
                if ((yys = yyname[YYTRANSLATE(yychar)]) == NULL) yys = yyname[YYUNDFTOKEN];
                fprintf(stderr, "%s[%d]: state %d, reading token %d (%s)\n",
                                YYDEBUGSTR, yydepth, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == YYEOF) goto yyaccept;
        goto yyloop;
    }
    if (((yyn = yygindex[yym]) != 0) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == (YYINT) yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
    {
        fprintf(stderr, "%s[%d]: after reduction, ", YYDEBUGSTR, yydepth);
#ifdef YYSTYPE_TOSTRING
#if YYBTYACC
        if (!yytrial)
#endif /* YYBTYACC */
            fprintf(stderr, "result is <%s>, ", YYSTYPE_TOSTRING(yystos[yystate], yyval));
#endif
        fprintf(stderr, "shifting from state %d to state %d\n", *yystack.s_mark, yystate);
    }
#endif
    if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack) == YYENOMEM) goto yyoverflow;
    *++yystack.s_mark = (YYINT) yystate;
    *++yystack.l_mark = yyval;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    *++yystack.p_mark = yyloc;
#endif
    goto yyloop;
#if YYBTYACC

    /* Reduction declares that this path is valid. Set yypath and do a full parse */
yyvalid:
    if (yypath) YYABORT;
    while (yyps->save)
    {
        YYParseState *save = yyps->save;
        yyps->save = save->save;
        save->save = yypath;
        yypath = save;
    }
#if YYDEBUG
    if (yydebug)
        fprintf(stderr, "%s[%d]: state %d, CONFLICT trial successful, backtracking to state %d, %d tokens\n",
                        YYDEBUGSTR, yydepth, yystate, yypath->state, (int)(yylvp - yylvals - yypath->lexeme));
#endif
    if (yyerrctx)
    {
        yyFreeState(yyerrctx);
        yyerrctx = NULL;
    }
    yylvp          = yylvals + yypath->lexeme;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yylpp          = yylpsns + yypath->lexeme;
#endif
    yylexp         = yylexemes + yypath->lexeme;
    yychar         = YYEMPTY;
    yystack.s_mark = yystack.s_base + (yypath->yystack.s_mark - yypath->yystack.s_base);
    memcpy (yystack.s_base, yypath->yystack.s_base, (size_t) (yystack.s_mark - yystack.s_base + 1) * sizeof(YYINT));
    yystack.l_mark = yystack.l_base + (yypath->yystack.l_mark - yypath->yystack.l_base);
    memcpy (yystack.l_base, yypath->yystack.l_base, (size_t) (yystack.l_mark - yystack.l_base + 1) * sizeof(YYSTYPE));
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
    yystack.p_mark = yystack.p_base + (yypath->yystack.p_mark - yypath->yystack.p_base);
    memcpy (yystack.p_base, yypath->yystack.p_base, (size_t) (yystack.p_mark - yystack.p_base + 1) * sizeof(YYLTYPE));
#endif
    yystate        = yypath->state;
    goto yyloop;
#endif /* YYBTYACC */

yyoverflow:
    YYERROR_CALL("yacc stack overflow");
#if YYBTYACC
    goto yyabort_nomem;
yyenomem:
    YYERROR_CALL("memory exhausted");
yyabort_nomem:
#endif /* YYBTYACC */
    yyresult = 2;
    goto yyreturn;

yyabort:
    yyresult = 1;
    goto yyreturn;

yyaccept:
#if YYBTYACC
    if (yyps->save) goto yyvalid;
#endif /* YYBTYACC */
    yyresult = 0;

yyreturn:
#if defined(YYDESTRUCT_CALL)
    if (yychar != YYEOF && yychar != YYEMPTY)
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        YYDESTRUCT_CALL("cleanup: discarding token", yychar, &yylval, &yylloc);
#else
        YYDESTRUCT_CALL("cleanup: discarding token", yychar, &yylval);
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */

    {
        YYSTYPE *pv;
#if defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED)
        YYLTYPE *pp;

        for (pv = yystack.l_base, pp = yystack.p_base; pv <= yystack.l_mark; ++pv, ++pp)
             YYDESTRUCT_CALL("cleanup: discarding state",
                             yystos[*(yystack.s_base + (pv - yystack.l_base))], pv, pp);
#else
        for (pv = yystack.l_base; pv <= yystack.l_mark; ++pv)
             YYDESTRUCT_CALL("cleanup: discarding state",
                             yystos[*(yystack.s_base + (pv - yystack.l_base))], pv);
#endif /* defined(YYLTYPE) || defined(YYLTYPE_IS_DECLARED) */
    }
#endif /* defined(YYDESTRUCT_CALL) */

#if YYBTYACC
    if (yyerrctx)
    {
        yyFreeState(yyerrctx);
        yyerrctx = NULL;
    }
    while (yyps)
    {
        YYParseState *save = yyps;
        yyps = save->save;
        save->save = NULL;
        yyFreeState(save);
    }
    while (yypath)
    {
        YYParseState *save = yypath;
        yypath = save->save;
        save->save = NULL;
        yyFreeState(save);
    }
#endif /* YYBTYACC */
    yyfreestack(&yystack);
    return (yyresult);
}
